package Configuration;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Configuration.pm
# Version       : 1.05
# Purpose       : Configuration module.
# Date          : 2003-07-28
# Updates       : 1.00 First release. (2003-01-05)
#                 1.01 Formatter initialization moved from  (2003-01-16)
#                        Formatter::Collection.
#                 1.02 Added run_time, -t. (2003-01-23)
#                 1.03 Added CSV formatter related code. (2003-01-27)
#                 1.04 Added url_word_break related code. (2003-01-28)
#                 1.04 Added rel_1_report, rel_2_report. (2003-07-28)
# Notes         : -
#
#=============================================================================

use strict;
use FileHandle;
use File::Basename;
use Formatter::Base;
use Formatter::Xhtml;
use Formatter::Html;
use Formatter::Text;
use Formatter::Textless;
use Formatter::CSV;
use Wordsplit;
use WordsplitGB;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.04 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter);
};




sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my ($self)  = {};

    bless ($self, $class);
    $self -> {"CONFIGURATION"} = $self -> read_configuration ();

    return $self;
};




sub get {
    my ($self) = shift;
    my ($item) = shift;

    return $self -> {"CONFIGURATION"} -> {$item};
    # return $self -> {"CONFIGURATION"} -> {$item} || undef;
};




sub get_all {
    my ($self) = shift;

    return $self -> {"CONFIGURATION"} || undef;
};




# Don't use
sub set {
    my ($self) = shift;
    my (%param) = @_;

    my ($key)   = undef;
    my ($value) = undef;

    # $self -> {"CONFIGURATION"} -> {$item} = $param {-value};

    foreach $key (keys (%param)) {
        $value = $param {$key};
        $self -> {"CONFIGURATION"} -> {$key} = $value;

        # print ($key, "\t", $param {$key}, "\t", $value, "\n");
    };
};




#-----------------------------------------------------------------------------
# Function parse_switches
#
# Purpose  : Parses command line.
#
# Input    : - (@ARGV)
# Output   : Hash, switch-value pairs.  Default values are used if the switch
#              is not given.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub parse_switches {
    my ($self) = shift;

    my (%switches) = (
        "ref_log_fname_list" => [],
        "report_filename"    => "-",
        "config_filename"    => "",
        "archive_filename"   => "",
        "read_archive_filename" => "",
        "html_format"        => 1, # deprecated, comment out
        "report_format"      => 'html',
        "local_urls"         => '.*',
        "lower_case_query_terms"=> 0,
        "charset"            => "",
        "exclude_targets"    => '',
        "exclude_refs"       => '',
        "unzip_app"          => {
            '(?i)\.gz$'  => 'gzip -dc',
            '(?i)\.bz2$' => 'bzip2 -dc',
            '(?i)\.zip$' => 'unzip -p',
            '(?i)\.Z$'   => 'uncompress -c',
        },
        "language"           => "en_US",
        "data_separator"     => ",",
        "decimal_point"      => ".",
        "word_splitter"      => "",
        "template_filename"  => "",
        "help"               => 0,
        "etsin"              => 0,
        "etsin_domain"       => "http://invalid/",
        "strip_dir"          => "", # deprecated
        "strip_params"       => 0,
        "split_reports"      => 0,
        "standardize_urls"   => 0,
        "skip_dead_engines"  => 0,
        "table_of_contents"  => 0,
        "url_word_break"     => 0,
        "run_time"           => 0,
        "nopercent"          => 0,
        "no_item_numbers"    => 0,
        "popup_window"       => 0,
        "progress_indicator" => 0,
        "file_permissions"   => undef,
        "all_reports"        => 0,
        "rgs_report"         => 0,
        "engine_report"      => 0,
        "rel_1_report"       => 0,
        "rel_2_report"       => 0,
        "rsq_1_report"       => 0,
        "rsq_2_report"       => 0,
        "rqs_1_report"       => 0,
        "rqs_2_report"       => 0,
        "query_report"       => 0,
        "keyword_report"     => 0,
        "version"            => 0,
        "rra_report"         => 0,
        "rrl_report"         => 0,
        "rrf_report"         => 0,
        "rrad_1_report"      => 0,
        "rrad_2_report"      => 0,
        "rrld_1_report"      => 0,
        "rrld_2_report"      => 0,
        "rrfd_1_report"      => 0,
        "rrfd_2_report"      => 0,
        "rqd_1_report"       => 0,
        "rqd_2_report"       => 0,
        "rkd_1_report"       => 0,
        "rkd_2_report"       => 0,
        "rtad_1_report"      => 0,
        "rtad_2_report"      => 0,
        "engine_list"        => 0,
        # "engine_list"        => "",
        "nickname_realname"  => {},
        "rewrite_referring_url" => {}
        #3 "log_end_of_line"    => $/ || "\n",
        #3 "log_format_apache"  => [],
        #3 "log_format_analog"  => []
    );
    my ($switch);

    foreach $switch (@ARGV) {
        # print ($switch, "\n");
        ($switch =~ /^([^\-].+)/) && (
            push (@{$switches {"ref_log_fname_list"}}, $1),
            next
        );
        ($switch =~ /^\-rgs/) && ($switches {"rgs_report"} = 1, next);
        ($switch =~ /^\-l(.+)/) && ($switches {"local_urls"} = $1, next);
        ($switch =~ /^\-x(.+)/) && ($switches {"exclude_targets"} = $1, next);
        ($switch =~ /^\-X(.+)/) && ($switches {"exclude_refs"} = $1, next);
        ($switch =~ /^\-o(.+)/) && ($switches {"report_filename"} = $1, next);
        ($switch =~ /^\-a(.+)/) && ($switches {"archive_filename"} = $1, next);
        ($switch =~ /^\-A(.+)/) && ($switches {"read_archive_filename"} = $1,
            next);
        ($switch =~ /^\-c(.+)/) && ($switches {"config_filename"} = $1, next);
        ($switch =~ /^\-T(.+)/) && ($switches {"template_filename"} = $1,next);
        ($switch =~ /^\-g(.+)/) && ($switches {"language"} = $1, next);
        ($switch =~ /^\-Rs(.+)/) && ($switches {"data_separator"} = $1, next);
        ($switch =~ /^\-Rp(.+)/) && ($switches {"decimal_point"} = $1, next);
        ($switch =~ /^\-f(.+)/) && ($switches {"file_permissions"} = $1, next);
        ($switch =~ /^\-ed(.+)/) && ($switches {"etsin_domain"} = $1, next);
        #3 ($switch =~ /^\-el(.+)/) && ($switches {"log_end_of_line"} = $1, next);
        ($switch =~ /^\-e/) && ($switches {"etsin"} = 1, next);
        # -html is deprecated, make it backward compatible (<=2.60)
        ($switch =~ /^\-html/) && ($switches {"report_format"} = 'html',next);
        #($switch =~ /^\-html/) && ($switches {"html_format"} = 1, next);
        ($switch =~ /^(?:\-h|\-\-help)/) && ($switches {"help"} = 1, next);
        # -sd is deprecated
        ($switch =~ /^\-sd(.+)/) && ($switches {"strip_dir"} = $1, next);
        ($switch =~ /^\-sp/) && ($switches {"strip_params"} = 1, next);
        ($switch =~ /^\-sr/) && ($switches {"split_reports"} = 1, next);
        ($switch =~ /^\-s/) && ($switches {"standardize_urls"} = 1, next);
        ($switch =~ /^\-S/) && ($switches {"skip_dead_engines"} = 1, next);
        ($switch =~ /^\-toc/) && ($switches {"table_of_contents"} = 1, next);
        ($switch =~ /^\-t/) && ($switches {"run_time"} = 1, next);
        ($switch =~ /^\-uwb/) && ($switches {"url_word_break"} = 1, next);
        ($switch =~ /^\-\-version/) && ($switches {"version"} = 1, next);
        ($switch =~ /^\-pw/) && ($switches {"popup_window"} = 1, next);
        # make -pi backward compatible (<=2.60)
        ($switch =~ /^\-pi$/) && ($switches {"progress_indicator"}=1,next);
        ($switch =~ /^\-pi=?(\d*)/) && (
            $switches {"progress_indicator"} = $1,
            next
        );
        ($switch =~ /^\-np/) && ($switches {"nopercent"} = 1, next);
        ($switch =~ /^\-ni/) && ($switches {"no_item_numbers"} = 1, next);
        ($switch =~ /^\-E(.?)/) && ($switches {"engine_list"} = 1, next);
        ($switch =~ /^\-CS=(.+)/) && ($switches {"charset"} = $1, next);
        ($switch =~ /^\-W=(.+)/) && ($switches {"word_splitter"} = $1, next);
        ($switch =~ /^\-C/) && ($switches {"lower_case_query_terms"} = 1,next);
        ($switch =~ /^\-F(.+)/) && (
            $switches {"report_format"} = lc ($1),
            next
        );

        #3($switch =~ /^\-Ln=?(.+)/) && (
        #    push (@{$switches {"log_format_analog"}}, {"REXP" => $1}),
        #    next
        #);
        #($switch =~ /^\-Lp=?(.+)/) && (
        #    push (@{$switches {"log_format_apache"}}, {"REXP" => $1}),
        #    next
        #);

        ($switch =~ /^\-N(.+)=(.+)/) && (
            $switches {"nickname_realname"}{$1} = $2,
            next
        );
        ($switch =~ /^\-wr(.+)=(.+)/) && (
            $switches {"rewrite_referring_url"}{$1} = $2,
            next
        );
        ($switch =~ /^\-z(.+)=(.+)/) && (
            $switches {"unzip_app"}{$1} = $2,
            next
        );
        # make -z backward compatible (<=2.3)
        ($switch =~ /^\-z(.+)/) && ($switches {"unzip_app"}{'\.gz$'}=$1, next);

        ($switch =~ /^\-ra(\-?\d+)/) && (
            $switches {"all_reports"}   =
            $switches {"rgs_report"}    =
            $switches {"engine_report"} =
            $switches {"rel_1_report"}  =
            $switches {"rel_2_report"}  =
            $switches {"rsq_1_report"}  =
            $switches {"rsq_2_report"}  =
            $switches {"rqs_1_report"}  =
            $switches {"rqs_2_report"}  =
            $switches {"query_report"}  =
            $switches {"keyword_report"}=
            $switches {"rra_report"}    =
            $switches {"rrl_report"}    =
            $switches {"rrf_report"}    =
            $switches {"rrad_1_report"} =
            $switches {"rrad_2_report"} =
            $switches {"rrld_1_report"} =
            $switches {"rrld_2_report"} =
            $switches {"rrfd_1_report"} =
            $switches {"rrfd_2_report"} =
            $switches {"rqd_1_report"}  =
            $switches {"rqd_2_report"}  =
            $switches {"rkd_1_report"}  =
            $switches {"rkd_2_report"}  =
            $switches {"rtad_1_report"} =
            $switches {"rtad_2_report"} = $1,
            next
        );

        ($switch =~ /^\-re(\-?\d+)/) && ($switches {"engine_report"}=$1, next);
        ($switch =~ /^\-rqd(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rqd_1_report"}, $switches {"rqd_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rel(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rel_1_report"}, $switches {"rel_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rsq(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rsq_1_report"}, $switches {"rsq_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rqs(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rqs_1_report"}, $switches {"rqs_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rq(\-?\d+)/) && ($switches {"query_report"} =$1, next);
        ($switch =~ /^\-rkd(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rkd_1_report"}, $switches {"rkd_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rk(\-?\d+)/) && ($switches {"keyword_report"}=$1,next);
        ($switch =~ /^\-rra(\-?\d+)/) && ($switches {"rra_report"} = $1, next);
        ($switch =~ /^\-rrl(\-?\d+)/) && ($switches {"rrl_report"} = $1, next);
        ($switch =~ /^\-rrf(\-?\d+)/) && ($switches {"rrf_report"} = $1, next);
        ($switch =~ /^\-rrad(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rrad_1_report"}, $switches {"rrad_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rrld(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rrld_1_report"}, $switches {"rrld_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rrfd(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rrfd_1_report"}, $switches {"rrfd_2_report"}) =
             ($1, $2), next);
        ($switch =~ /^\-rtad(\-?\d+)\,(\-?\d+)/) &&
            (($switches {"rtad_1_report"}, $switches {"rtad_2_report"}) =
             ($1, $2), next);
    };

    if (scalar (@{$switches {"ref_log_fname_list"}}) == 0) {
        push (@{$switches {"ref_log_fname_list"}}, "-") # stdin
    };

    #3 $switches {"log_end_of_line"} =~ s|\\(0\d+)|chr (oct ($1))|ge;

    return %switches;
};




sub read_configuration {
    my ($self) = shift;

    my (%command_line) = $self -> parse_switches ();
    my ($default_config_file) = "default.cfg";
    my ($foo) = $File::Basename::VERSION; # just to get rid of warning...
    my ($relax_name, $relax_path, $relax_suffix)= fileparse ($0, '\.[^.]*');

    if (-e $relax_path . $default_config_file) {
        eval ($self -> slurp_file ($relax_path . $default_config_file));
        if ($@) {
            $self -> warn_and_die (
                "Invalid configuration file: ",
                $relax_path . $default_config_file,
                $@
            )
        };
    };

    if ($command_line {"config_filename"} ne "") {
        eval ($self -> slurp_file ($command_line {"config_filename"}));
        if ($@) {
            $self -> warn_and_die (
                "Invalid configuration file: ",
                $command_line {"config_filename"},
                $@
            )
        };
    };

    if ($command_line {"report_format"} =~ /^xhtml$/i) {
        $command_line {"formatter"} = Formatter::Xhtml -> new ();
    }
    elsif ($command_line {"report_format"} =~ /^html$/i) {
        $command_line {"formatter"} = Formatter::Html -> new ();
    }
    elsif ($command_line {"report_format"} =~ /^text$/i) {
        $command_line {"formatter"} = Formatter::Text -> new ();
    }
    elsif ($command_line {"report_format"} =~ /^textless$/i) {
        $command_line {"formatter"} = Formatter::Textless -> new ();
    }
    elsif ($command_line {"report_format"} =~ /^csv$/i) {
        $command_line {"formatter"} = Formatter::CSV -> new (
            -data_separator => $command_line {"data_separator"},
            -decimal_point  => $command_line {"decimal_point"},
        );
    };

    if (! $command_line {"formatter"}) {
        $self -> warn_and_die (
            "Invalid report format: ",
            $command_line {"report_format"}.
            $@
        );
    };

    $command_line {"report_template"} = "";
    if ($command_line {"template_filename"} ne "") {
        $command_line {"report_template"} = $self -> slurp_file (
            $command_line {"template_filename"}
        );
    }
    else {
        $command_line {"report_template"} =
            $command_line {"formatter"} -> template ();
    };

    $command_line {"relax_path"} = $relax_path;

    if ($command_line {"progress_indicator"}) {
        $command_line {"progress_indicator"} *= 1000;
    };

    if ($command_line {"word_splitter"} =~ /GB/i) {
        $command_line {"word_splitter_obj"} = WordsplitGB -> new ();
    }
    else {
        $command_line {"word_splitter_obj"} = Wordsplit -> new ();
    };

    $command_line {"text_cell_width_n_column"} = 6;
    $command_line {"text_cell_width_count_column"} = 8;
    $command_line {"text_cell_width_pct_column"} = 7;
    $command_line {"text_cell_width_mainhead_column"} = 53;
    # In General summary report
    $command_line {"text_cell_width_wide_mainhead_column"} = 61;
    # In Table of contents and Supported search engines report
    $command_line {"text_cell_width_extra_wide_mainhead_column"} = 78;
    # text_cell_width_n_column + text_cell_width_list_item = 
    #   text_cell_width_extra_wide_mainhead_column
    $command_line {"text_cell_width_list_item"} = 72;

    return { %command_line };
};




#-----------------------------------------------------------------------------
# Function slurp_file
#
# Purpose  : Reads a whole file into a string.
#
# Input    : $source_filename, String, Name of the file.
# Output   : String
# Error
# handling : Dies if the file cannot be opened.
#
#-----------------------------------------------------------------------------
sub slurp_file {
    my ($self)      = shift;
    my ($file_name) = shift;

    my ($fh) = FileHandle -> new ();
    my ($data);
    local ($/) = undef; # input_record_separator

    if ($fh -> open ($file_name)) {
        # $fh -> input_record_separator (undef);
        $data = $fh -> getline ();
    }
    else {
        $self -> warn_and_die ("Cannot open " . $file_name);
    };

    $fh -> close;

    return $data;
};




sub warn_and_die {
    my ($self)     = shift;
    my (@messages) = @_;

    die ($0 , ": ", join ("\n", @messages), "\n");
};




sub warn_and_continue {
    my ($self)     = shift;
    my (@messages) = @_;

    warn ($0 , ": ", join ("\n", @messages), "\n");
};




1;

