package Formatter::Xhtml;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Formatter::Xhtml.pm
# Version       : 1.03
# Purpose       : Formats reports to XHTML format.
# Date          : 2003-09-08
# Updates       : 1.00 First release. (2002-12-17)
#                 1.01 &apos; entity changed to &#39; (2003-01-28)
#                 1.02 Removed unnecessary quotation marks from  (2003-07-20)
#                        meta charset.
#                 1.03 Print stylesheet improved.  (2003-09-08)
# Notes         : -
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.03 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter Formatter::Base);
};




sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my $self    = $class -> SUPER::new (@_);

    bless ($self, $class);

    return $self;
};


sub begin_division {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (
        '<div' .
        ($self -> id () ? ' id="' . $self -> id () . '"' : '') .
        ($param {-class} ? ' class="' . $param {-class} . '"' : '') .
        ">\n"
    );
};


sub end_division {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</div>\n\n")
};


sub begin_table {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (
        '<table' .
        ($self -> summary () ?
            ' summary="' . $self -> html_ent ($self -> summary ()) . '"' :
            ''
        ) .
        ($self -> summary () ?
            ' title="' . $self -> html_ent ($self -> summary ()) . '"' :
            ''
        ) .
        ($param {-class} ? ' class="' . $param {-class} . '"' : '') .
        ">\n"
    );
};


sub end_table {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</table>\n");
};


sub begin_caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (
        '<caption' .
        ($param {-class} ? ' class="' . $param {-class} . '"' : '') .
        ">"
    );
};


sub caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> html_ent ($param {-value}));
};


sub end_caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</caption>\n");
};


sub begin_table_header {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("<thead>\n");
};


sub end_table_header {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</thead>\n");
};


sub begin_table_body {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("<tbody>\n");
};


sub end_table_body {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</tbody>\n");
};


sub begin_row {
    my ($self)  = shift;
    my (%param) = @_;

    my ($class_str) = $param {-class} ?
        ' class="' . lc ($param {-class}) . '"' :
        "";

    return $self -> add_to_content_str ('<tr' . $class_str . '>');
};


sub end_row {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</tr>\n");
};


sub begin_header_row {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> begin_row (%param);
};


sub end_header_row {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> end_row (%param);
};


sub begin_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($align) = $param {-align} ?
        ' align="' . lc ($param {-align}) . '"' :
        "";

    return $self -> add_to_content_str ('<td' . $align . '>');
};


sub end_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ('</td>');
};


sub begin_header_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($align) = $param {-align} ?
        ' align="' . lc ($param {-align}) . '"' :
        "";
    my ($class) = $param {-class} ?
        ' class="' . $param {-class} . '"' :
        "";

    return $self -> add_to_content_str ('<th' . $align . $class . '>');
};


sub end_header_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ('</th>');
};



sub begin_list {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("<ul>\n");
};


sub end_list {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</ul>\n");
};


sub begin_list_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ('<li>');
};


sub end_list_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("</li>\n");
};


sub begin_heading {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ('<h' . ($param {-level} || '1') . '>');
};


sub end_heading {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ('</h' . ($param {-level} || '1'). '>');
};


sub heading_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> html_ent ($param {-value}));
};


sub begin_paragraph {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (
        '<p' .
        ($param {-class} ? ' class="' . $param {-class} . '"' : '') .
        ">\n"
    );
};


sub end_paragraph {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n</p>\n");
};


sub text_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> html_ent ($param {-value}));
};


sub item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (
        $param {-value} ? $self -> html_ent ($param {-value}) : '&nbsp;'
    );
};


sub header_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> item (%param);
};


sub url_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($url_str) = "";
    my ($url_text) = "";

    if (!$param {-no_chk} && $param {-url} !~ /
            ^
            (?:
                (http|ftp).*?\/\/
                (?:[\w\d\-]+[\.\@\:]|\d{4,})
                .*?\/
                |
                \#[\w\d\-]+
            )
        /ix
    ) {
        $url_str = $self -> html_ent ($param {-url});
    }
    else {
        $url_text = $self -> html_ent ($param {-url_text});
        if ($param {-word_break}) {
            # <wbr> is not part of XHTML standard
            $url_text =~ s/([\/\?\-\ ])/$1\<wbr \/\>/g;
        };

        $url_str = '<a href="' .
            $self -> html_ent ($param {-url}) . '"' .
            ($param {-target} ? ' target="' . $param {-target} . '"' : '') .
            '>' .
            $url_text .
        '</a>'
    };

    return $self -> add_to_content_str ($url_str);
};


sub css_stylesheet {
    my ($self)  = shift;
    my (%param) = @_;

    return <<END
        body {
            background: white;
            color: black;
            margin-left: 5%;
            margin-right: 5%;
        }

        A:link {
            color: #000066;
            background: transparent;
        }
        A:active {
            color: #ff0000;
            background: transparent;
        }
        A:visited {
            color: #0066cc;
            background: transparent;
        }

        .document-navigation {
            margin-bottom: 2em;
        }

        .document-footer {
            border-top: solid #cccccc;
        }

        .report {
            margin-bottom: 3em;
        }

        .report-table {
            border: solid #cccccc;
            width: 100%;
        }

        .report-table td {
            vertical-align:top;
        }

        .report-table caption {
            text-align: left;
            font-weight: bolder;
            font-size: larger;
        }

        .report-mainhead {
            color: black;
            background: #999999;
        }

        .rsh {
            /* report-subhead */
            color: black;
            background: #cccccc;
            font-weight: bolder;
        }

        .n-col {
            width: 5%;
        }

        .count-col {
            width: 5%;
        }

        .pct-col {
            width: 10%;
        }
END
};


sub css_print_stylesheet {
    my ($self)  = shift;
    my (%param) = @_;

    return <<END
        a:link, a:visited, a:active {
            text-decoration: none;
            color: black;
            background: white;
        }

        .document-navigation {
            display: none;
        }

        h1 {
            font-size: xx-large;
        }

        .report {
            margin-bottom: 3em;
        }

        .report-table td {
            vertical-align: top;
        }

        .report-table caption {
            text-align: left;
            font-size: x-large;
        }

        .report-mainhead {
            color: black;
            background: white;
            font-size: larger;
            font-weight: bolder;
        }

        .rsh {
            /* report-subhead */
            color: black;
            background: white;
            font-weight: bolder;
        }

        .n-col {
            width: 5%;
        }

        .count-col {
            width: 5%;
        }

        .pct-col {
            width: 10%;
        }
END
};


sub template {
    my ($self)  = shift;
    my (%param) = @_;

    my ($stylesheet) = $self -> css_stylesheet ();
    my ($print_stylesheet) = $self -> css_print_stylesheet ();

    return <<END
<?xml version="1.0" encoding="<!--[charset]-->"?>
<!DOCTYPE html 
    PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="<!--[report_language]-->" lang="<!--[report_language]-->">
  <head>
    <title><!--[report_title]--></title>
    <link href="mailto:ktmatu\@surfeu.fi.remove.invalid" rev="made" />
    <meta http-equiv="Content-Type" content="text/html; charset=<!--[charset]-->" />
    <meta name="description" content="Relax Referrer Log Analysis Report" />
    <meta name="keywords" content="Relax" />
    <meta name="robots" content="noindex,nofollow" />
    <meta name="DC.Date.Created" content="<!--[date]-->" />
    <meta name="DC.Creator" content="<!--[relax_version]-->" />
    <meta name="useless" content="meta tag" />
    <style type="text/css" media="screen">
    <!--
$stylesheet
    -->
    </style>
    <style type="text/css" media="print">
    <!--
$print_stylesheet
    -->
    </style>
  </head>
  <body>
<!--[report_header]-->
<!--[report_content]-->
<!--[report_navigation]-->
<!--[report_footer]-->
  </body>
</html>
END
};


#-----------------------------------------------------------------------------
# Function html_ent
#
# Purpose  : Converts dangerous characters to html entities.
#
# Input    : $str, string.
# Output   : String
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub html_ent {
    my ($self) = shift;
    my ($str)  = shift;

    sub encode {
        my ($char) = shift;

        my (%char_ent) = (
            "&" => "&amp;",
            "<" => "&lt;",
            ">" => "&gt;",
            '"' => "&quot;",
            "'" => "&#39;", # "&apos;" apos is not a valid entity
        );

        return $char_ent {$char};
    };

    $str =~ s/([\Q&><"'\E])/encode ($1)/ge;

    return $str;
};



#-----------------------------------------------------------------------------
# Function url_encode
#
# Purpose  : Converts dangerous URL characters to %xx format.
#
# Input    : $str, string.
# Output   : String
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub url_encode {
    my ($self) = shift;
    my ($str)  = shift;

    $str =~ s/([^a-zA-Z0-9 ])/sprintf "%%%lx", ord ($1)/ge;
    $str =~ s/([ ])/\+/g;

    return $str;
};



sub escape_meta_data {
    my ($self)  = shift;
    my ($str) = @_;

    return $self -> html_ent ($str);
};




1;
