package Reports;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Reports.pm
# Version       : 1.00
# Purpose       : Report generation related methods.
# Date          : 2003-07-30
# Updates       : 1.00 First release. (2003-07-30)
#
# Notes         : -
#
#=============================================================================

use strict;
use File::Basename;
use TimeDate;
use Strftime;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.00 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter);
};




sub new {
    my ($proto) = shift;
    my (%param) = @_;
    my ($class) = ref ($proto) || $proto;
    my ($self)  = {};

    bless ($self, $class);

    return $self;
};




#-----------------------------------------------------------------------------
# Function sum_hash
#
# Purpose  : Adds up values in a hash of key-value pairs.
#
# Input    : $hash, ref to a hash.
#            $keys, ref to a list which contains keys to items to be added.
# Output   : scalar.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub sum_hash {
    my ($self) = shift;
    my ($hash, $keys) = @_;
    my ($item);
    my ($sum) = 0;

    foreach $item (@$keys) {
        $sum += $hash -> {$item};
    };

    return $sum;
};




#-----------------------------------------------------------------------------
# Function rev_numeric_sort_hash_by_value
#
# Purpose  : Performs reverse numeric sort by value to a hash of
#              key-value pairs.
#
# Input    : $hash, ref to a hash.
# Output   : reference to an array of hash keys.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub rev_numeric_sort_hash_by_value {
    my ($self) = shift;
    my ($hash) = @_;

    return [
        sort {
            $hash -> {$b} <=> $hash -> {$a} ||
            $a cmp $b
        } keys (%$hash)
    ]
};




#-----------------------------------------------------------------------------
# Function make_header
#
# Purpose  : Creates report document's header.
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub make_header {
    my ($self) = shift;
    my (
        $formatter,
        $messages,
        $config
    ) = @_;

    $formatter -> begin_report (
        -id           => "header",
        -report_title => "",
        -summary      => "",
    );

    $formatter -> begin_heading (-level => "1");
    $formatter -> heading_item (
        -value => $messages -> get ("relax_report"),
        -level => "1",
    );
    $formatter -> end_heading (-level => "1");

    $formatter -> end_report ();
};




#-----------------------------------------------------------------------------
# Function make_footer
#
# Purpose  : Creates report document's footer.
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
#            $relax_version, String, Relax release number ("1.23").
#            $engines_parser_version, String, Engines.pm version
#              number ("1.23")
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub make_footer {
    my ($self) = shift;
    my (
        $formatter,
        $messages,
        $config,
        $relax_version,
        $engines_parser_version
    ) = @_;

    $formatter -> begin_report (
        -id           => "footer",
        -report_title => "",
        -summary      => "",
    );

    $formatter -> begin_paragraph (-class => "document-footer");
    $formatter -> text_item (-value => $messages -> get ("generated_by") ." ");
    $formatter -> url_item (
        -url      => "http://ktmatu.com/software/relax/",
        -url_text => $relax_version,
    );
    $formatter -> text_item (-value => " + " . $messages -> get("parser")." ");
    $formatter -> text_item (-value => $engines_parser_version);
    $formatter -> end_paragraph ();

    $formatter -> end_report ();
};




#-----------------------------------------------------------------------------
# Function make_navigation
#
# Purpose  : Creates report document's navigation elements ("Previous",
#              "Table of Contents", "Next".
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
#            $toc, ref to list, List of report name ids ("rk", "rra" ...).
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub make_navigation {
    my ($self) = shift;
    my (
        $formatter,
        $messages,
        $config,
        $toc
    ) = @_;

    my (@toc_lol) = @{$toc};
    my ($i);
    my ($next) = undef;
    my ($prev) = undef;
    my ($file_base, $file_path, $file_type) = fileparse (
        $config -> get ("report_filename"),
        '\.[^.]*'
    );

    for $i (0 .. $#toc_lol) {
        $prev = ($i == 0 ? undef : $toc_lol [$i - 1]);
        $next = ($i == $#toc_lol ? undef : $toc_lol [$i + 1]);

        $formatter -> begin_report (
            -id           => "navigation-" . $toc_lol [$i],
            -report_title => "",
            -summary      => "",
        );

        $formatter -> begin_paragraph (-class => "document-navigation");

        if ($prev) {
            $formatter -> url_item (
                -url      => ($prev eq "toc" ?
                                 $file_base . $file_type :
                                 $file_base . "-" . $prev . $file_type),
                -url_text => $messages -> get ("Previous"),
                -no_chk   => 1,
            );
        }
        else {
            $formatter -> text_item (-value => $messages -> get ("Previous"));
        };

        $formatter -> text_item (-value => " | ");

        if ($toc_lol [$i] eq "toc") {
            $formatter -> text_item (
                -value => $messages -> get ("table_of_contents")
            );
        }
        else {
            $formatter -> url_item (
                -url      => $file_base . $file_type,
                -url_text => $messages -> get ("table_of_contents"),
                -no_chk   => 1,
            );
        };

        $formatter -> text_item (-value => " | ");

        if ($next) {
            $formatter -> url_item (
                -url      => $file_base . "-" . $next . $file_type,
                -url_text => $messages -> get ("Next"),
                -no_chk   => 1,
            );
        }
        else {
            $formatter -> text_item (-value => $messages -> get ("Next"));
        };

        $formatter -> end_paragraph ();
        $formatter -> end_report ();
    };
};




#-----------------------------------------------------------------------------
# Function make_one_level_report
#
# Purpose  : Creates one level report (value/key).
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
#            $hash, ref to a hash.
#            $keys, ref to a list which contains hash keys.
#            $min, integer, discard items less than num in value from report.
#            $item_anchor_type, integer,
#              1 = create an HTML anchor out of item name
#              2 = create Etsin anchor
#              3 = create a link to a search engine
#            $report_id, string,  report name id ("rk", "rra" ...).
#            $report_title, string, report title ("Search engine keywords")
#            $report_summary, string, long description of the report.
#            $table_mainhead, string, title of the main head in the
#              report table.
#            $engine_info, ref to hash, information about search engines.
# Output   : string
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub make_one_level_report {
    my ($self) = shift;
    my (
        $formatter,
        $messages,
        $config,
        $hash,
        $keys,
        $min,
        $item_anchor_type,
        $etsin_domain,
        $report_id,
        $report_title,
        $report_summary,
        $table_mainhead,
        $engine_info
    )= @_;

    my ($item);
    my ($group_count) = 0;
    my ($group_ref_count) = 0;
    my ($counter) = 0;
    my ($sum) = $self -> sum_hash ($hash, $keys);
    my ($nopercent) = $config -> get ("nopercent");
    my ($no_item_numbers) = $config -> get ("no_item_numbers");
    my ($percent_str) = "";
    my ($target_win_str) = $config -> get ("popup_window") ? "r" : "";
    my ($link_str) = "";
    my ($class_str) = undef;
    my ($n_column_width) = $config -> get ("text_cell_width_n_column");
    my ($count_column_width) = $config -> get ("text_cell_width_count_column");
    my ($pct_column_width) = $config -> get ("text_cell_width_pct_column");
    my ($mainhead_column_width) =
        $config -> get ("text_cell_width_mainhead_column");
    my ($url_word_break) = $config -> get ("url_word_break");


    $formatter -> begin_report (
        -id           => $report_id,
        -report_title => $report_title,
        -summary      => $report_summary
    );

    $formatter -> begin_division (
        -id    => $report_id,
        -class => "report",
        -align => "center"
    );
    $formatter -> begin_table (-class => "report-table");

    $formatter -> begin_caption (-class => "report-caption");
    $formatter -> caption (
        -value => $report_title,
        -align => "left",
    );
    $formatter -> end_caption ();

    $formatter -> begin_table_header ();
    $formatter -> begin_header_row (-class => "report-mainhead");

        if (! $no_item_numbers) {
            $formatter -> begin_header_item (
                -length => $n_column_width,
                -align  => "center",
                -class  => "n-col",
            );
            $formatter -> header_item (
                -value  => $messages -> get ("n."),
                -length => $n_column_width,
                -align  => "center",
            );
            $formatter -> end_header_item ();
        };

        $formatter -> begin_header_item (
            -align  => "center",
            -length => $count_column_width,
            -class  => "count-col"
        );
        $formatter -> header_item (
            -value  => $messages -> get ("count"),
            -length => $count_column_width,
            -align  => "center",
        );
        $formatter -> end_header_item ();


        if (! $nopercent) {
            $formatter -> begin_header_item (
                -length => $pct_column_width,
                -align  => "center",
                -class  => "pct-col",
            );
            $formatter -> header_item (
                -value  => "%",
                -length => $pct_column_width,
                -align  => "center",
            );
            $formatter -> end_header_item ();
        };

        $formatter -> begin_header_item (
            -length => $mainhead_column_width,
        );
        $formatter -> header_item (
            -value  => $table_mainhead || $report_title,
            -length => $mainhead_column_width,
        );
        $formatter -> end_header_item ();

    $formatter -> end_header_row ();

    $formatter -> end_table_header ();

    $formatter -> begin_table_body ();

    foreach $item (@$keys) {
        $counter++;

        if (
            (($min >= 0) && ($hash -> {$item} >= $min)) ||
            (($min < 0) && ($counter <= abs ($min)))
            ) {
            $class_str = undef;
            $formatter -> begin_row (-class => $class_str);

            if (! $no_item_numbers) {
                $formatter -> begin_item (
                    -length => $n_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> item (
                    -value  => $counter,
                    -length => $n_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> end_item ();
            };

            $formatter -> begin_item (
                -length => $count_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => $hash -> {$item},
                -length => $count_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();


            $percent_str = "";
            if (! $nopercent) {
                $percent_str = 
                     sprintf ('%.2f', (100 * $hash -> {$item}) / $sum) . "%";

                $formatter -> begin_item (
                    -length => $pct_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> item (
                    -value => $percent_str,
                    -length => $pct_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> end_item ();
            };


            $formatter -> begin_item (
                -length => $mainhead_column_width,
                -class  => $class_str,
            );

            if ($item_anchor_type == 1) {
                $formatter -> url_item (
                    -url      => $item,
                    -url_text => $item,
                    -target   => $target_win_str,
                    -length   => $mainhead_column_width,
                    -align    => "left",
                    -class    => $class_str,
                    -word_break => $url_word_break,
                );
            }
            elsif ($item_anchor_type == 2) {
                $formatter -> url_item (
                    -url      => $self -> create_etsin_url (
                                     $item,
                                     $etsin_domain,undef
                                 ),
                    -url_text => $item,
                    -target   => $target_win_str,
                    -length   => $mainhead_column_width,
                    -align    => "left",
                    -class    => $class_str,
                );
            }
            elsif ($item_anchor_type == 3) {
                $formatter -> url_item (
                    -url      => $engine_info -> {$item}{"url"},
                    -url_text => $item,
                    -target   => $target_win_str,
                    -length   => $mainhead_column_width,
                    -align    => "left",
                    -class    => $class_str,
                );
            }
            else {
                $formatter -> item (
                    -value  => $item,
                    -length => $mainhead_column_width,
                    -align  => "left",
                    -class  => $class_str,
                );
            };

            $formatter -> end_item ();

            $formatter -> end_row ();
        }
        else {
            $group_count++;
            $group_ref_count += $hash -> {$item};
        };
    };

    if ($group_count > 0) {
        $class_str = "rsh";
        $formatter -> begin_row (-class => $class_str);

        if (! $no_item_numbers) {
            $formatter -> begin_item (
                -length => $n_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => "",
                -length => $n_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();
        };

        $formatter -> begin_item (
            -length => $count_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => "",
            -length => $count_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();

        if (! $nopercent) {
            $formatter -> begin_item (
                -length => $pct_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => "",
                -length => $pct_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();
        };

        $formatter -> begin_item (
            -length => $mainhead_column_width,
            -class  => $class_str,
        );

        $formatter -> item (
            -value  => "..." . $group_count . " (" . $group_ref_count . ")",
            -length => $mainhead_column_width,
            -align  => "left",
            -class  => $class_str,
        );

        $formatter -> end_item ();

        $formatter -> end_row ();
    };

    $class_str = "report-mainhead";
    $formatter -> begin_row (-class => $class_str);

    if (! $no_item_numbers) {
        $formatter -> begin_item (
            -length => $n_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => "",
            -length => $n_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();
    };

    $formatter -> begin_item (
        -length => $count_column_width,
        -align  => "right",
        -class  => $class_str,
    );
    $formatter -> item (
        -value  => $sum,
        -length => $count_column_width,
        -align  => "right",
        -class  => $class_str,
    );
    $formatter -> end_item ();

    if (! $nopercent) {
        $formatter -> begin_item (
            -length => $pct_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => "",
            -length => $pct_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();
    };

    $formatter -> begin_item (
        -length => $mainhead_column_width,
        -class  => $class_str,
    );

    $formatter -> item (
        -value  => $messages -> get ("total"),
        -length => $mainhead_column_width,
        -align  => "left",
        -class  => $class_str,
    );

    $formatter -> end_item ();

    $formatter -> end_row ();


    $formatter -> end_table_body ();
    $formatter -> end_table ();
    $formatter -> end_division ();

    $formatter -> end_report ();

    return 1;
};




#-----------------------------------------------------------------------------
# Function make_two_level_report
#
# Purpose  : Creates two level report (value/key -> targets).
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
#            $otherref_count, ref to a hash.
#            $ref_target_count, ref to a hash.
#            $group_keys, ref to list, keys to first level items.
#            $group_min, integer, discard first level items if less than this.
#            $item_min,integer, discard second level items if less than this.
#            $group_anchor_type, integer,
#              0 = do nothing,
#              1 = create an HTML anchor out of group name,
#              2 = Etsin anchor
#            $item_anchor_type, integer,
#              0 = do nothing
#              1 = create an HTML anchor out of item name
#              5 = create a link to a search engine
#            $base_host_url, string, site's http://www.example.com part of the
#              URL
#            $base_path_url, string, site's /~ktmatu/ part of the
#              URL
#            $report_id, string,  report name id ("rk", "rra" ...).
#            $report_title, string, report title ("Search engine keywords")
#            $report_summary, string, long description of the report.
#            $table_mainhead, string, title of the main head in the
#              report table
#            $engine_info, ref to hash, information about search engines.
# Output   : string
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub make_two_level_report {
    my ($self) = shift;
    my (
        $formatter,
        $messages,
        $config,
        $otherref_count,
        $ref_target_count,
        $group_keys,
        $group_min,
        $item_min,
        $group_anchor_type,
        $item_anchor_type,
        $base_host_url,
        $base_path_url,
        $report_id,
        $report_title,
        $report_summary,
        $table_mainhead,
        $engine_info
    ) = @_;

    my ($group, $subgroup);
    my (@group_keys);
    my (@keys);
    my ($keys_ref);
    my ($count) = 0;
    my ($hit_count) = 0;
    my ($group_count) = 0;
    my ($group_hit_count) = 0;
    my ($group_counter) = 0;
    my ($item_counter) = 0;
    my ($nopercent) = $config -> get ("nopercent");
    my ($no_item_numbers) = $config -> get ("no_item_numbers");
    my ($percent_str) = "";
    my ($sum) = $self -> sum_hash ($otherref_count, $group_keys);
    my ($target_win_str) = $config -> get ("popup_window") ? "r" : "";
    my ($class_str) = undef;
    my ($n_column_width) = $config -> get ("text_cell_width_n_column");
    my ($count_column_width) = $config -> get ("text_cell_width_count_column");
    my ($pct_column_width) = $config -> get ("text_cell_width_pct_column");
    my ($mainhead_column_width) =
        $config -> get ("text_cell_width_mainhead_column");
    my ($url_word_break) = $config -> get ("url_word_break");

    $formatter -> begin_report (
        -id           => $report_id,
        -report_title => $report_title,
        -summary      => $report_summary
    );

    $formatter -> begin_division (
        -id    => $report_id,
        -class => "report",
        -align => "center",
    );
    $formatter -> begin_table (-class => "report-table");

    $formatter -> begin_caption (-class => "report-caption");
    $formatter -> caption (
        -value => $report_title,
        -align => "left",
    );
    $formatter -> end_caption ();

    $formatter -> begin_table_header ();
    $formatter -> begin_header_row (-class => "report-mainhead");

        if (! $no_item_numbers) {
            $formatter -> begin_header_item (
                -length => $n_column_width,
                -align  => "center",
                -class  => "n-col",
            );
            $formatter -> header_item (
                -value  => $messages -> get ("n."),
                -length => $n_column_width,
                -align  => "center",
            );
            $formatter -> end_header_item ();
        };

        $formatter -> begin_header_item (
            -align  => "center",
            -length => $count_column_width,
            -class  => "count-col",
        );
        $formatter -> header_item (
            -value  => $messages -> get ("count"),
            -length => $count_column_width,
            -align  => "center",
        );
        $formatter -> end_header_item ();


        if (! $nopercent) {
            $formatter -> begin_header_item (
                -length => $pct_column_width,
                -align  => "center",
                -class  => "pct-col",
            );
            $formatter -> header_item (
                -value  => "%",
                -length => $pct_column_width,
                -align  => "center",
            );
            $formatter -> end_header_item ();
        };

        $formatter -> begin_header_item (
            -length => $mainhead_column_width,
        );
        $formatter -> header_item (
            -value  => $table_mainhead || $report_title,
            -length => $mainhead_column_width,
        );
        $formatter -> end_header_item ();

    $formatter -> end_header_row ();

    $formatter -> end_table_header ();

    $formatter -> begin_table_body ();


    foreach $group (@$group_keys) {
        $group_counter++;

        if (
            (($group_min >= 0) && ($otherref_count -> {$group} >= $group_min))
            ||
            (($group_min < 0) && ($group_counter <= abs ($group_min)))
        ) {
            $class_str = "rsh";

            $formatter -> begin_row (-class => $class_str);

            if (! $no_item_numbers) {
                $formatter -> begin_item (
                    -length => $n_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> item (
                    -value  => $group_counter,
                    -length => $n_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> end_item ();
            };

            $formatter -> begin_item (
                -length => $count_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => $otherref_count -> {$group},
                -length => $count_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();


            $percent_str = "";
            if (! $nopercent) {
                $percent_str =
                    sprintf ('%.2f', (100 * $otherref_count->{$group}) / $sum).
                    "%";

                $formatter -> begin_item (
                    -length => $pct_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> item (
                    -value  => $percent_str,
                    -length => $pct_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> end_item ();
            };


            $formatter -> begin_item (
                -length => $mainhead_column_width,
                -class  => $class_str,
            );

            if ($group_anchor_type == 1) {
                $formatter -> url_item (
                    -url      => $group,
                    -url_text => $group,
                    -target   => $target_win_str,
                    -length   => $mainhead_column_width,
                    -align    => "left",
                    -class    => $class_str,
                    -word_break => $url_word_break,
                );
            }
            elsif ($group_anchor_type == 2) {
                $formatter -> url_item (
                    -url => $self -> create_etsin_url (
                                $group,
                                $base_host_url . $base_path_url,
                                undef
                            ),
                    -url_text => $group,
                    -target   => $target_win_str,
                    -length   => $mainhead_column_width,
                    -align    => "left",
                    -class    => $class_str,
                );
            }
            elsif ($group_anchor_type == 3) {
                $formatter -> url_item (
                    -url      => $engine_info -> {$group}{"url"},
                    -url_text => $group,
                    -target   => $target_win_str,
                    -length   => $mainhead_column_width,
                    -align    => "left",
                    -class    => $class_str,
                );
            }
            else {
                $formatter -> item (
                    -value  => $group,
                    -length => $mainhead_column_width,
                    -align  => "left",
                    -class  => $class_str,
                );
            };

            $formatter -> end_item ();

            $formatter -> end_row ();

            $keys_ref = $self -> rev_numeric_sort_hash_by_value (
                $ref_target_count -> {$group}
            );
            $count = 0;
            $hit_count = 0;
            $item_counter = 0;

            for $subgroup (@$keys_ref) {
                $class_str = undef;
                $item_counter++;
                if (
                    (
                        ($item_min >= 0) &&
                        ($ref_target_count -> {$group}{$subgroup} >= $item_min)
                    )
                    ||
                    (
                        ($item_min < 0) &&
                        ($item_counter <= abs ($item_min))
                    )
                ) {
                    if ($nopercent) {
                        $percent_str = "";
                    }
                    else {
                        $percent_str = sprintf ('%.2f', (100 *
                            $ref_target_count -> {$group}{$subgroup}) /
			    $otherref_count -> {$group}) . "%"
                    };

                    $formatter -> begin_row (-class => $class_str);

                    if (! $no_item_numbers) {
                        $formatter -> begin_item (
                            -length => $n_column_width,
                            -align  => "right",
                            -class  => $class_str,
                        );
                        $formatter -> item (
                            -value  => $item_counter,
                            -length => $n_column_width,
                            -align  => "right",
                            -class  => $class_str,
                        );
                        $formatter -> end_item ();
                    };

                    $formatter -> begin_item (
                        -length => $count_column_width,
                        -align  => "right",
                        -class  => $class_str,
                    );
                    $formatter -> item (
                        -value  => $ref_target_count ->{$group}{$subgroup},
                        -length => $count_column_width,
                        -align  => "right",
                        -class  => $class_str,
                    );
                    $formatter -> end_item ();


                    if (! $nopercent) {
                        $formatter -> begin_item (
                            -length => $pct_column_width,
                            -align  => "right",
                            -class  => $class_str,
                        );
                        $formatter -> item (
                            -value => $percent_str,
                            -length => $pct_column_width,
                            -align  => "right",
                            -class  => $class_str,
                        );
                        $formatter -> end_item ();
                    };

                    $formatter -> begin_item (
                        -length => $mainhead_column_width,
                        -class  => $class_str,
                    );
                    if ($item_anchor_type == 1) {
                        $formatter -> url_item (
                            -url      => $subgroup,
                            -url_text => $subgroup,
                            -target   => $target_win_str,
                            -length   => $mainhead_column_width,
                            -align    => "left",
                            -class    => $class_str,
                            -word_break => $url_word_break,
                        );
                    }
                    elsif ($item_anchor_type == 2) {
                        $formatter -> url_item (
                            -url      => $self -> create_etsin_url (
                                             $subgroup,
                                             $base_host_url . $base_path_url,
                                             undef
                                         ),
                            -url_text => $subgroup,
                            -target   => $target_win_str,
                            -length   => $mainhead_column_width,
                            -align    => "left",
                            -class    => $class_str,
                        );
                    }
                    elsif ($item_anchor_type == 3) {
                        $formatter -> url_item (
                            -url      => $self -> create_etsin_url (
                                             $subgroup,
                                             $base_host_url . $base_path_url,
                                             $group
                                         ),
                            -url_text => $subgroup,
                            -target   => $target_win_str,
                            -length   => $mainhead_column_width,
                            -align    => "left",
                            -class    => $class_str,
                        );
                    }
                    elsif ($item_anchor_type == 4) {
                        $formatter -> url_item (
                            -url      => $self -> create_etsin_url (
                                             $group,
                                             $base_host_url . $subgroup,
                                             undef
                                         ),
                            -url_text => $subgroup,
                            -target   => $target_win_str,
                            -length   => $mainhead_column_width,
                            -align    => "left",
                            -class    => $class_str,
                            -word_break => $url_word_break,
                        );
                    }
                    elsif ($item_anchor_type == 5) {
                        $formatter -> url_item (
                            -url      => $engine_info -> {$subgroup}{"url"},
                            -url_text => $subgroup,
                            -target   => $target_win_str,
                            -length   => $mainhead_column_width,
                            -align    => "left",
                            -class    => $class_str,
                            -word_break => $url_word_break,
                        );
                    }
                    else {
                        $formatter -> item (
                            -value  => $subgroup,
                            -length => $mainhead_column_width,
                            -align  => "left",
                            -class  => $class_str,
                        );
                    };
                    $formatter -> end_item ();

                    $formatter -> end_row ();
                }
                else {
                    $count++;
                    $hit_count += $ref_target_count -> {$group}{$subgroup};
                }
            }
            if ($count > 0) {
                $class_str = undef;
                $formatter -> begin_row (-class => $class_str);

                if (! $no_item_numbers) {
                    $formatter -> begin_item (
                        -length => $n_column_width,
                        -align  => "right",
                        -class  => $class_str,
                    );
                    $formatter -> item (
                        -value  => "",
                        -length => $n_column_width,
                        -align  => "right",
                        -class  => $class_str,
                    );
                    $formatter -> end_item ();
                };

                $formatter -> begin_item (
                    -length => $count_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> item (
                    -value  => "",
                    -length => $count_column_width,
                    -align  => "right",
                    -class  => $class_str,
                );
                $formatter -> end_item ();

                if (! $nopercent) {
                    $formatter -> begin_item (
                        -length => $pct_column_width,
                        -align  => "right",
                        -class  => $class_str,
                    );
                    $formatter -> item (
                        -value  => "",
                        -length => $pct_column_width,
                        -align  => "right",
                        -class  => $class_str,
                    );
                    $formatter -> end_item ();
                };

                $formatter -> begin_item (
                    -length => $mainhead_column_width,
                    -class  => $class_str,
                );

                $formatter -> item (
                    -value  => "..." . $count . " (" . $hit_count. ")",
                    -length => $mainhead_column_width,
                    -align  => "left",
                    -class  => $class_str,
                );

                $formatter -> end_item ();

                $formatter -> end_row ();
            }
        }
        else {
            $group_count++;
            $group_hit_count += $otherref_count -> {$group};
        };
    };
    if ($group_count > 0) {
        $class_str = "rsh";
        $formatter -> begin_row (-class => $class_str);

        if (! $no_item_numbers) {
            $formatter -> begin_item (
                -length => $n_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => "",
                -length => $n_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();
        };

        $formatter -> begin_item (
            -length => $count_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => "",
            -length => $count_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();

        if (! $nopercent) {
            $formatter -> begin_item (
                -length => $pct_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => "",
                -length => $pct_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();
        };

        $formatter -> begin_item (
            -length => $mainhead_column_width,
            -class  => $class_str,
        );

        $formatter -> item (
            -value  => "..." . $group_count . " (" . $group_hit_count. ")",
            -length => $mainhead_column_width,
            -align  => "left",
            -class  => $class_str,
        );

        $formatter -> end_item ();

        $formatter -> end_row ();
    };


    $class_str = "report-mainhead";
    $formatter -> begin_row (-class => $class_str);

    if (! $no_item_numbers) {
        $formatter -> begin_item (
            -length => $n_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => "",
            -length => $n_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();
    };

    $formatter -> begin_item (
        -length => $count_column_width,
        -align  => "right",
        -class  => $class_str,
    );
    $formatter -> item (
        -value  => $sum,
        -length => $count_column_width,
        -align  => "right",
        -class  => $class_str,
    );
    $formatter -> end_item ();

    if (! $nopercent) {
        $formatter -> begin_item (
            -length => $pct_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => "",
            -length => $pct_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();
    };

    $formatter -> begin_item (
        -length => $mainhead_column_width,
        -class  => $class_str,
    );

    $formatter -> item (
        -value  => $messages -> get ("total"),
        -length => $mainhead_column_width,
        -align  => "left",
        -class  => $class_str,
    );

    $formatter -> end_item ();

    $formatter -> end_row ();


    $formatter -> end_table_body ();
    $formatter -> end_table ();
    $formatter -> end_division ();

    $formatter -> end_report ();

    return 1;
};




#-----------------------------------------------------------------------------
# Function url_encode
#
# Purpose  : Converts dangerous URL characters to %xx format.
#
# Input    : $str, string.
# Output   : String
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub url_encode {
    my ($self) = shift;
    my ($str) = @_;

    $str =~ s/([^a-zA-Z0-9 ])/sprintf "%%%lx", ord ($1)/ge;
    $str =~ s/([ ])/\+/g;

    return $str;
};




#-----------------------------------------------------------------------------
# Function general_summary_one_row
#
# Purpose  : Creates one row to general statistics report.
#
# Input    : $value, integer, value in "Count" column.
#            $percent, string, value in "%" column.
#            $description, string, value in "Description" column.
#            $class_str, string, "class" of the row/column.
#            $formatter, Formatter object.
#            $config, Configuration object.
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub general_summary_one_row {
    my ($self) = shift;
    my (
        $value,
        $percent,
        $description,
        $class_str,
        $formatter,
        $config
    ) = @_;

    my ($n_column_width) = $config -> get ("text_cell_width_n_column");
    my ($count_column_width) = $config -> get ("text_cell_width_count_column");
    my ($pct_column_width) = $config -> get ("text_cell_width_pct_column");
    my ($mainhead_column_width) =
        $config -> get ("text_cell_width_wide_mainhead_column");

    $formatter -> begin_row (-class => $class_str);
        $formatter -> begin_item (
            -length => $count_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => $value,
            -length => $count_column_width,
            -align  => "right",
            -class  => $class_str,
        );
        $formatter -> end_item ();

        if (! $config -> get ("nopercent")) {
            $formatter -> begin_item (
                -length => $pct_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> item (
                -value  => $percent,
                -length => $pct_column_width,
                -align  => "right",
                -class  => $class_str,
            );
            $formatter -> end_item ();
        };

        $formatter -> begin_item (
            -length => $mainhead_column_width,
            -align  => "left",
            -class  => $class_str,
        );
        $formatter -> item (
            -value  => $description,
            -length => $mainhead_column_width,
            -align  => "left",
            -class  => $class_str,
        );
        $formatter -> end_item ();

    $formatter -> end_row ();
}



#-----------------------------------------------------------------------------
# Function general_statistics_report
#
# Purpose  : Creates general statistics report.
#
# Input    : $stat_hash, ref to a hash.
#            $general_stat_hash, ref to a hash.
#            $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
# Output   : string
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub general_statistics_report {
    my ($self) = shift;
    my (
        $stat_hash,
        $general_stat_hash,
        $formatter,
        $messages,
        $config
    ) = @_;

    my ($unknown_refs) = 0;
    my ($sum) = 0;
    my ($nopercent) = $config -> get ("nopercent");
    my ($class_str) = "";
    my ($n_column_width) = $config -> get ("text_cell_width_n_column");
    my ($count_column_width) = $config -> get ("text_cell_width_count_column");
    my ($pct_column_width) = $config -> get ("text_cell_width_pct_column");
    my ($mainhead_column_width) =
        $config -> get ("text_cell_width_wide_mainhead_column");

    if ($stat_hash -> {"unknown_foreign_refs"} > 0) {
        $stat_hash -> {"count_foreign_referring_urls"} -=
            $stat_hash -> {"unknown_foreign_refs"};
        $stat_hash -> {"uniq_foreign_referring_urls"}--;
        $unknown_refs = $stat_hash -> {"unknown_foreign_refs"};
    }
    elsif ($stat_hash -> {"unknown_local_refs"} > 0) {
        $stat_hash -> {"count_local_referring_urls"} -=
            $stat_hash -> {"unknown_local_refs"};
        $stat_hash -> {"uniq_local_referring_urls"}--;
        $unknown_refs = $stat_hash -> {"unknown_local_refs"};
    };

    $sum = $stat_hash -> {"sengine_refs"} +
        $stat_hash -> {"count_foreign_referring_urls"} +
        $stat_hash -> {"count_local_referring_urls"} +
        $unknown_refs;

    $formatter -> begin_report (
        -id           => "rgs",
        -report_title => $messages -> get ("general_summary"),
        -summary      => $messages -> get ("general_summary_summary...")
    );

    $formatter -> begin_division (
        -id    => "rgs",
        -class => "report",
        -align => "center"
    );
    $formatter -> begin_table (-class => "report-table");

    $formatter -> begin_caption (-class => "report-caption");
    $formatter -> caption (
        -value => $messages -> get ("general_summary"),
        -align => "left",
    );
    $formatter -> end_caption ();

    $formatter -> begin_table_header ();
    $formatter -> begin_header_row (-class => "report-mainhead");
        $formatter -> begin_header_item (
            -length => $count_column_width,
            -align  => "center",
            #-class  => "n-col",
        );
        $formatter -> header_item (
            -value  => $messages -> get ("count"),
            -length => $count_column_width,
            -align  => "center",
        );
        $formatter -> end_header_item ();

        if (! $nopercent) {
            $formatter -> begin_header_item (
                -length => $pct_column_width,
                -align  => "center",
                -class  => "pct-col",
            );
            $formatter -> header_item (
                -value  => "%",
                -length => $pct_column_width,
                -align  => "center",
            );
            $formatter -> end_header_item ();
        };

        $formatter -> begin_header_item (
            -length => $mainhead_column_width,
        );
        $formatter -> header_item (
            -value  => $messages -> get ("referral_type"),
            -length => $mainhead_column_width,
        );
        $formatter -> end_header_item ();

    $formatter -> end_header_row ();

    $formatter -> end_table_header ();

    $formatter -> begin_table_body ();
        $self -> general_summary_one_row (
            $stat_hash -> {"sengine_refs"},
            $self ->percent ($stat_hash -> {"sengine_refs"}, $sum, $nopercent),
            $messages -> get ("referred_by_search_engines"),
            "rsh",
            $formatter,
            $config
        );

        $self -> general_summary_one_row (
            $stat_hash -> {"uniq_sengines"},
            undef,
            $messages -> get ("unique_search_engines"),
            undef,
            $formatter,
            $config
        );

        $self -> general_summary_one_row (
            $stat_hash -> {"uniq_queries"},
            undef,
            $messages -> get ("unique_query_terms"),
            undef,
            $formatter,
            $config
        );

        $self -> general_summary_one_row (
            $stat_hash -> {"uniq_keywords"},
            undef,
            $messages -> get ("unique_keywords"),
            undef,
            $formatter,
            $config
        );

    $self -> general_summary_one_row (
        $stat_hash -> {"total_keywords"},
        undef,
        $messages -> get ("keyword_count"),
        $class_str,
        $formatter,
        $config
    );

    $self -> general_summary_one_row (
        $stat_hash -> {"count_foreign_referring_urls"},
        $self -> percent (
            $stat_hash -> {"count_foreign_referring_urls"}, $sum, $nopercent),
        $messages -> get ("referred_by_foreign_urls"),
        "rsh",
        $formatter,
        $config
    );

    $self -> general_summary_one_row (
        $stat_hash -> {"uniq_foreign_referring_urls"},
        undef,
        $messages -> get ("unique_urls"),
        undef,
        $formatter,
        $config
    );

    $self -> general_summary_one_row (
        $stat_hash -> {"count_local_referring_urls"},
        $self -> percent (
            $stat_hash -> {"count_local_referring_urls"}, $sum, $nopercent),
        $messages -> get ("referred_by_local_urls"),
        "rsh",
        $formatter,
        $config
    );

    $self -> general_summary_one_row (
        $stat_hash -> {"uniq_local_referring_urls"},
        undef,
        $messages -> get ("unique_urls"),
        undef,
        $formatter,
        $config
    );

    $self -> general_summary_one_row (
        $unknown_refs,
        $self -> percent ($unknown_refs, $sum, $nopercent),
        $messages -> get ("unknown_direct_from_bookmarks_typed_urls"),
        "rsh",
        $formatter,
        $config
    );

    $self -> general_summary_one_row (
        $sum,
        undef,
        "(+ " . $messages -> get ("excluded_targets") . ": " .
            ($general_stat_hash -> {"target_excluded"} ||  0) .
            " + " . $messages -> get ("excluded_referrers") . ": " .
            ($general_stat_hash -> {"referrer_excluded"} ||  0) .
            " + " . $messages -> get ("corrupted_lines") . ": " .
            ($general_stat_hash -> {"corrupted_lines"} ||  0) .
            " = " .
            ($general_stat_hash -> {"lines_in_log"} || 0) .
            ")",
        "report-mainhead",
        $formatter,
        $config
    );

    $formatter -> end_table_body ();
    $formatter -> end_table ();
    $formatter -> end_division ();

    $formatter -> end_report ();

    return 1;
};




#-----------------------------------------------------------------------------
# Function percent
#
# Purpose  : Reurns the ratio of two numbers as a formatted string.
#
# Input    : $val1, Real, dividend.
#            $val2, Real, divisor.
#            $nopercent, Boolean, 1 = return an empty string instead of
#              formatted percent.
# Output   : String
# Error
# handling : Returns an empty string in case of val2 = 0.
#
#-----------------------------------------------------------------------------
sub percent {
    my ($self) = shift;
    my ($val1, $val2, $nopercent) = @_;

    my ($percent_str) = "";

    if (($val2 > 0) && (not ($nopercent))) {
        $percent_str = sprintf (
            '%.2f',
            100 * ($val1 / $val2)
        ) .
        "%";
    };

    return $percent_str;
};




#-----------------------------------------------------------------------------
# Function create_etsin_url
#
# Purpose  : Creates Etsin URL.
#
# Input    : $keyword_str, string, keyword, kw=... .
#            $etsin_domain_str, string, url=... .
#            $search_engine_str, string, name of the search engine, se=... .
# Output   : String
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub create_etsin_url {
    my ($self) = shift;
    my (
        $keyword_str,
        $etsin_domain_str,
        $search_engine_str
    ) = @_;

    my ($result_str) = "";
    my ($kw_str)  = "";
    my ($url_str) = "";
    my ($se_str)  = "";

    if (defined ($keyword_str) and ($keyword_str ne "")) {
        $kw_str = "kw=" . $self -> url_encode ($keyword_str) . "&";
    };

    if (defined ($etsin_domain_str) and ($etsin_domain_str ne "")) {
        $url_str = "url=" . $self -> url_encode ($etsin_domain_str) . "&";
    };

    if (defined ($search_engine_str) and ($search_engine_str ne "")) {
        $se_str = "se=" . $self -> url_encode ($search_engine_str) . "&";
    };

    $result_str = 'http://ktmatu.com/cgi-bin/etsin.pl' .
        "?" .
        $kw_str .
        $url_str .
        $se_str;

    return $result_str;
};




#-----------------------------------------------------------------------------
# Function engine_list_report
#
# Purpose  : Creates engine list report. This is mainly for testing purposes.
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
#            $engine_info, ref to hash, information about search engines.
# Output   : string
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub engine_list_report {
    my ($self) = shift;
    my (
        $formatter,
        $messages,
        $config,
        $engine_info
    ) = @_;

    my ($item);
    my ($target_win_str) = $config -> get ("popup_window") ? "r" : "";
    my ($n_column_width) = $config -> get ("text_cell_width_n_column");
    my ($mainhead_column_width) =
        $config -> get ("text_cell_width_extra_wide_mainhead_column");
    my ($list_item_width) = $config -> get ("text_cell_width_list_item");

    $formatter -> begin_report (
        -id           => "el",
        -report_title => $messages -> get ("supported_search_engines"),
        -summary      => $messages ->
                             get ("supported_search_engines_summary..."),
    );

    $formatter -> begin_division (
        -id    => "el",
        -class => "report",
        -align => "center"
    );

    $formatter -> begin_table (-class => "report-table");
    $formatter -> begin_caption (-class => "report-caption");
    $formatter -> caption (
        -value => $messages -> get ("supported_search_engines"),
        -align => "left",
    );
    $formatter -> end_caption ();

    $formatter -> begin_table_header ();
    $formatter -> begin_header_row (-class => "report-mainhead");

        $formatter -> begin_header_item (
            -length => $mainhead_column_width,
            -align  => "center",
        );
        $formatter -> header_item (
            -value  => $messages -> get ("search_engine"),
            -length => $mainhead_column_width,
            -align  => "center",
        );
        $formatter -> end_header_item ();

    $formatter -> end_header_row ();

    $formatter -> end_table_header ();

    $formatter -> begin_table_body ();

    $formatter -> begin_row ();

    $formatter -> begin_item (
        -align  => "left",
        -length => $n_column_width #???
    );

    $formatter -> begin_list ();

    foreach $item (sort (keys (%{$engine_info}))) {
        $formatter -> begin_list_item ();

        if ($engine_info -> {$item}{"url"} =~ /\.invalid\//i) {
            $formatter -> item (
                -value    => $item,
                -length   => $list_item_width,
            );
        }
        else {
            $formatter -> url_item (
                -url      => $engine_info -> {$item}{"url"},
                -url_text => $item,
                -no_chk   => 1,
                -target   => $target_win_str,
                -length   => $list_item_width,
            );
        };

        $formatter -> end_list_item ();
    };

    $formatter -> end_list ();

    $formatter -> end_item ();

    $formatter -> end_row ();


    $formatter -> begin_row (
        -class => "report-mainhead"
    );

    $formatter -> begin_item (
        -length => $list_item_width,
        -class  => "report-mainhead"
    );

    $formatter -> item (
        -value  => scalar (keys (%{$engine_info})) . " " .
                   $messages -> get ("total"),
        -length => $list_item_width,
        -align  => "left",
        -class  => "report-mainhead"
    );

    $formatter -> end_item ();

    $formatter -> end_row ();


    $formatter -> end_table_body ();
    $formatter -> end_table ();

    $formatter -> end_division ();

    $formatter -> end_report ();

    return 1;
};




#-----------------------------------------------------------------------------
# Function create_table_of_contents
#
# Purpose  : Creates report's table of content.
#
# Input    : $content_lol, List of Lists, list of (report name, report id
#              pairs.
#            $formatter, Formatter object.
#            $messages, Translated messages object.
#            $config, Configuration object.
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub create_table_of_contents {
    my ($self) = shift;
    my (
        $content_lol,
        $formatter,
        $messages,
        $config
    ) = @_;

    my (@toc_lol) = @$content_lol;
    my ($target_win_str) = "";
    # my ($target_win_str) = $config -> get ("popup_window") ? "r" : "";
    my ($i);
    my ($file_base, $file_path, $file_type) = fileparse (
        $config -> get ("report_filename"),
        '\.[^.]*'
    );
    my ($url) = "";
    my ($n_column_width) = $config -> get ("text_cell_width_n_column");
    my ($mainhead_column_width) =
        $config -> get ("text_cell_width_extra_wide_mainhead_column");
    my ($list_item_width) = $config -> get ("text_cell_width_list_item");

    $formatter -> begin_report (
        -id           => "toc",
        -report_title => $messages -> get ("table_of_contents"),
        -summary      => $messages -> get ("table_of_contents_summary...")
    );

    $formatter -> begin_division (
        -id    => "toc",
        -class => "report",
        -align => "center"
    );

    $formatter -> begin_table (-class => "report-table");
    $formatter -> begin_caption (-class => "report-caption");
    $formatter -> caption (
        -value => $messages -> get ("table_of_contents"),
        -align => "left",
    );
    $formatter -> end_caption ();

    $formatter -> begin_table_header ();
    $formatter -> begin_header_row (-class => "report-mainhead");

        $formatter -> begin_header_item (
            -length => $mainhead_column_width,
            -align  => "center",
        );
        $formatter -> header_item (
            -value  => $messages -> get ("available_reports"),
            -length => $mainhead_column_width,
            -align  => "center",
        );
        $formatter -> end_header_item ();

    $formatter -> end_header_row ();

    $formatter -> end_table_header ();

    $formatter -> begin_table_body ();

    $formatter -> begin_row (); #-class => $class_str);

    $formatter -> begin_item (
        -length => $n_column_width,
        -align  => "left",
        #-class  => $class_str,
    );

    $formatter -> begin_list ();
    for $i (0 .. $#toc_lol) {
        $formatter -> begin_list_item ();

        $url = "#" . $toc_lol [$i][1];
        if ($config -> get ("split_reports")) {
            $url = $file_base . "-" . $toc_lol [$i][1] . $file_type
        };

        $formatter -> url_item (
            -url      => $url,
            -url_text => $toc_lol [$i][0],
            -no_chk   => 1,
            -target   => $target_win_str,
            -length   => $list_item_width,
            -align    => "left",
        );
        $formatter -> end_list_item ();
    };
    $formatter -> end_list ();

    $formatter -> end_item ();

    $formatter -> end_row ();

    $formatter -> end_table_body ();
    $formatter -> end_table ();

    $formatter -> end_division ();

    $formatter -> end_report ();


    return 1;
};




#-----------------------------------------------------------------------------
# Function store_document
#
# Purpose  : Stores a document to the selected output device.
#
# Input    : $document_filename, string, name of the output file. "-" = stdout.
#            $document_str, String, Contents of the document to be stored.
#            $permissions, Octal number (0..), permissions of the file.
# Output   : -
# Error
# handling : Dies if the output file cannot be opened.
#
#-----------------------------------------------------------------------------
sub store_document {
    my ($self) = shift;
    my (
        $document_filename,
        $document_str,
        $permissions
    ) = @_;

    open (OUTPUT, ">" . $document_filename) ||
        die ("$0: " . "Cannot open " . $document_filename . "\n");

    (print OUTPUT $document_str);

    close (OUTPUT);

    $self -> change_file_permissions (
        $document_filename,
        $permissions
    );
};




#-----------------------------------------------------------------------------
# Function construct_document
#
# Purpose  : Construcs a document from a set of reports and other parts.
#
# Input    : $formatter, Formatter object.
#            $messages, Translated messages object.
#            $strftime, Date and time formatter object.
#            $config, Configuration object.
#            $content_lol, Ref to List of Lists, list of
#              (report name, report id pairs.
#            $engines_parser_version, String, Engines.pm version
#              number ("1.23")
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub construct_document {
    my ($self) = shift;
    my (
        $formatter,
        $strftime,
        $messages,
        $config,
        $content_lol,
        $relax_version,
        $engines_parser_version
    ) = @_;

    my ($report_id)    = "";
    my ($document_str) = "";
    my ($template_str) = $config -> get ("report_template");
    my ($charset)      = $config -> get ("charset") || "ISO-8859-1";
    my ($language)     = $formatter -> language () || "en-US";
    my ($title)        = $formatter -> title ();
    my ($date)         = "";
    my ($report_filename) = $config -> get ("report_filename");
    my (@toc_lol) = @$content_lol;
    my (@id_list) = ();
    my ($i);
    my ($report_header) = "";
    my ($report_footer) = "";
    my ($report_navigation) = "";
    my ($file_base, $file_path, $file_type) = fileparse (
        $report_filename,
        '\.[^.]*'
    );
    my ($date_format_str)    = "";
    my ($formatted_date_str) = "";
    my ($date_obj)           = TimeDate -> new ();
    my ($time_format_obj)    = $strftime;

    if ($config ->get ("table_of_contents") or $config->get ("split_reports")){
        push (@id_list, "toc");
    };

    for $i (0 .. $#toc_lol) {
        push (@id_list, $toc_lol [$i][1]);
    };

    $self -> make_navigation ($formatter, $messages, $config, \@id_list);

    if ($formatter -> streamable_format ()) {
        $self -> make_header ($formatter, $messages, $config);
        $self -> make_footer (
            $formatter,
            $messages,
            $config,
            $relax_version,
            $engines_parser_version
        );
        $report_header = $formatter -> get_report ("header") -> {"report"};
        $report_footer = $formatter -> get_report ("footer") -> {"report"};

        $charset  = $formatter -> escape_meta_data ($charset);
        $language = $formatter -> escape_meta_data ($language);

        foreach $report_id (@id_list) {
            $date = $time_format_obj -> format_time (
                $date_obj -> get_current_time_and_date (),
                '%Y-%m-%dT%H:%M:%S'
            );

            if ($config -> get ("split_reports")) {
                $title = $formatter -> title () . " - " .
                    $formatter -> get_report ($report_id) -> {"report_title"};
                $template_str = $config -> get ("report_template");
                $report_filename =
                    $file_path . $file_base . "-" . $report_id . $file_type;
                if ($report_id eq "toc") {
                    $report_filename = $config -> get ("report_filename");
                };

                $report_navigation = $formatter ->
                    get_report ("navigation-" . $report_id) -> {"report"};

                $document_str = "";
            };

            $title = $formatter -> escape_meta_data ($title);
            $date  = $formatter -> escape_meta_data ($date);

            $document_str .=
                $formatter -> get_report ($report_id) -> {"report"}
                || ($report_id . " Missing");

            if ($template_str =~/\Q<!--\E\s*?\Q[date:\E([^\]]+)\]\s*?\Q-->\E/){
                $date_format_str = $1;
                $formatted_date_str = $time_format_obj -> format_time (
                    $date_obj -> get_current_time_and_date (),
                    $date_format_str
                );
            };

            if ($config -> get ("split_reports")) {
                $template_str =~ s/\Q<!--[relax_version]-->\E/$relax_version/ge;
                $template_str =~ s/\Q<!--[date]-->\E/$date/ge;
                $template_str =~
                    s/\Q<!--[date:\E.*?\Q]-->\E/$formatted_date_str/ge;
                $template_str =~ s/\Q<!--[charset]-->\E/$charset/ge;
                $template_str =~ s/\Q<!--[report_language]-->\E/$language/ge;
                $template_str =~ s/\Q<!--[report_title]-->\E/$title/ge;
                $template_str =~
                    s/\Q<!--[report_content]-->\E/$document_str/ge;
                $template_str =~ s/\Q<!--[report_header]-->\E/$report_header/g;
                $template_str =~
                    s/\Q<!--[report_navigation]-->\E/$report_navigation/g;
                $template_str =~ s/\Q<!--[report_footer]-->\E/$report_footer/g;

                $self -> store_document (
                    $report_filename,
                    $template_str,
                    $config -> get ("file_permissions")
                );
            };
        };

        if (! $config -> get ("split_reports")) {
            $template_str =~ s/\Q<!--[relax_version]-->\E/$relax_version/ge;
            $template_str =~ s/\Q<!--[date]-->\E/$date/ge;
            $template_str =~
                s/\Q<!--[date:\E.*?\Q]-->\E/$formatted_date_str/ge;
            $template_str =~ s/\Q<!--[charset]-->\E/$charset/ge;
            $template_str =~ s/\Q<!--[report_language]-->\E/$language/ge;
            $template_str =~ s/\Q<!--[report_title]-->\E/$title/ge;
            $template_str =~ s/\Q<!--[report_content]-->\E/$document_str/ge;
            $template_str =~ s/\Q<!--[report_header]-->\E/$report_header/g;
            $template_str =~ s/\Q<!--[report_navigation]-->\E//g;
            $template_str =~ s/\Q<!--[report_footer]-->\E/$report_footer/g;

            $self -> store_document (
                $report_filename,
                $template_str,
                $config -> get ("file_permissions")
            );
        };
    };
};




#-----------------------------------------------------------------------------
# Function change_file_permissions
#
# Purpose  : Changes the permissions of a file.
#
# Input    : $filename_str, string, name of the output file. "-" = stdout.
#            $permissions, *Octal* number (0..), permissions of the file.
# Output   : -
# Error
# handling : Gives a warning if the permissions cannot be changed.
#
#-----------------------------------------------------------------------------
sub change_file_permissions {
    my ($self) = shift;
    my ($filename_str, $permissions) = @_;

    if (($filename_str ne "-") and defined ($permissions)) {
        $permissions = oct ($permissions) if $permissions =~ /^0/;
        chmod ($permissions, $filename_str) ||
            warn ("$0: Cannot change permissions " . $filename_str);
    };
};




1;
