package Strftime;

#=============================================================================
# Copyright (C) 1997, 2001, 2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# Module name   : Strftime.pm
# Version       : 1.10
# Purpose       : A class that implements various methods to convert date
#                   and time to string.
# Date          : 2003-07-17
# Updates       : 1.00 Original version  (1997-06-16)
#                 1.01 format_time now properly handles %W  (1997-06-26)
#                        and %U
#                 1.02 The code is now strict safe. (2001-12-13)
#                 1.10 Support for day and month name  (2003-07-17)
#                         localization added.
#
# References    : -
#
# Notes         : -
#
#=============================================================================

use TimeDate;
use strict;




#-----------------------------------------------------------------------------
# Function new
#
# Purpose  : Object constructor. English day and month names are used if
#              no parameters are given.
#
# Input    : $abbr_wd, ref to array, Abbreviated weekday names.
#            $full_wd, ref to array, Full weekday names.
#            $abbr_month, ref to array, Abbreviated month names.
#            $full_month, ref to array, Full month names.
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my ($self)  = {};
    my ($abbr_wd)    = shift;
    my ($full_wd)    = shift;
    my ($abbr_month) = shift;
    my ($full_month) = shift;

    $self -> {DATE_OBJ} = TimeDate -> new ();

    if ($abbr_wd) {
	# print ("***", @{$abbr_wd}, "***\n");
        $self -> {WDAY_ABBR} = {
            1 => @{$abbr_wd} [0],
            2 => @{$abbr_wd} [1],
            3 => @{$abbr_wd} [2],
            4 => @{$abbr_wd} [3],
            5 => @{$abbr_wd} [4],
            6 => @{$abbr_wd} [5],
            7 => @{$abbr_wd} [6],
        };
    }
    else {
        $self -> {WDAY_ABBR} = {
            1 => "Mon", 2 => "Tue", 3 => "Wed", 4 => "Thu",
            5 => "Fri", 6 => "Sat", 7 => "Sun"
        };
    };


    if ($full_wd) {
        $self -> {WDAY_FULL} = {
            1 => @{$full_wd} [0],
            2 => @{$full_wd} [1],
            3 => @{$full_wd} [2],
            4 => @{$full_wd} [3],
            5 => @{$full_wd} [4],
            6 => @{$full_wd} [5],
            7 => @{$full_wd} [6],
        };
    }
    else {
        $self -> {WDAY_FULL} = {
            1 => "Monday", 2 => "Tuesday",  3 => "Wednesday", 4 => "Thursday",
            5 => "Friday", 6 => "Saturday", 7 => "Sunday"
        };
    };

    if ($abbr_month) {
        $self -> {MONTH_ABBR} = {
            1 => @{$abbr_month} [0],
            2 => @{$abbr_month} [1],
            3 => @{$abbr_month} [2],
            4 => @{$abbr_month} [3],
            5 => @{$abbr_month} [4],
            6 => @{$abbr_month} [5],
            7 => @{$abbr_month} [6],
            8 => @{$abbr_month} [7],
            9 => @{$abbr_month} [8],
            10 => @{$abbr_month} [9],
            11 => @{$abbr_month} [10],
            12 => @{$abbr_month} [11]
        };
    }
    else {
        $self -> {MONTH_ABBR} = {
            1 => "Jan", 2 => "Feb", 3 => "Mar", 4 => "Apr",
            5 => "May", 6 => "Jun", 7 => "Jul", 8 => "Aug",
            9 => "Sep", 10=> "Oct", 11=> "Nov", 12=> "Dec"
        };
    };


    if ($full_month) {
        $self -> {MONTH_FULL} = {
            1 => @{$full_month} [0],
            2 => @{$full_month} [1],
            3 => @{$full_month} [2],
            4 => @{$full_month} [3],
            5 => @{$full_month} [4],
            6 => @{$full_month} [5],
            7 => @{$full_month} [6],
            8 => @{$full_month} [7],
            9 => @{$full_month} [8],
            10 => @{$full_month} [9],
            11 => @{$full_month} [10],
            12 => @{$full_month} [11]
        };
    }
    else {
        $self -> {MONTH_FULL} = {
            1 => "January",    2 => "February",  3 => "March",
            4 => "April",      5 => "May",       6 => "June",
            7 => "July",       8 => "August",    9 => "September",
            10 => "October",  11 => "November", 12 => "December"
        };
    };


    bless ($self, $class);
    return ($self);
};




#-----------------------------------------------------------------------------
# Function format_time
#
# Purpose  : Returns a string where all Field Descriptors are replaced by
#              their corresponding values.  All other characters are copied
#              to the returned string without modifications.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
#            $format_str, String, that contains field descriptors and other
#              text.  These descriptors are the same as in Unix date command.
#              The following field descriptors are replaced:
#               %%  same as % (note, there is a bug related to this!)
#               %a  abbreviated weekday name
#               %A  full weekday name
#               %b  abbreviated month name
#               %B  full month name
#               %d  day of month - 01 to 31
#               %D  date as %m/%d/%y
#               %e  day of month - 1 to 31 (single digits  are  preceded
#                   by a blank)
#               %h  abbreviated month name (alias for %b)
#               %H  hour - 00 to 23
#               %I  hour - 01 to 12
#               %j  day of year - 001 to 366
#               %k  hour - " 0" to "23"
#               %l  hour - " 0" to "12"
#               %m  month of year - 01 to 12
#               %M  minute - 00 to 59
#               %n  insert a new-line character
#               %p  string containing ante-meridiem or post-meridiem indicator
#                   (by default, AM or PM)
#               %r  time as %I:%M:%S %p
#               %R  time as %H:%M
#               %S  second - 00 to 60
#               %t  insert a tab character
#               %T  time as %H:%M:%S
#               %u  weekday as a decimal number - 1 (Sunday) to 7
#               %U  Week number as a decimal number 00 - 53 when Sunday =
#                   first day of the week
#               %V  ISO Week number as a decimal number 01 - 53
#               %w  weekday as a decimal number - 0 (Sunday) to 6
#               %W  Week number as a decimal number 00 - 53 when Monday =
#                   first day of the week
#               %x  %D, should be read from locale
#               %X  %T, should be read from locale
#               %y  year within century - 00 to 99
#               %Y  year as ccyy (4 digits)
#               %Z  timezone name
#
#               %Zu weekday as a decimal number - 1 (Monday) to 7 (Sunday)
#               %Zw weekday as a decimal number - 0 (Monday) to 6 (Sunday)
# Output   : String
# Error
# handling : -
# Bugs     : Does not correctly handle "%%u" type $format_strings
#            maybe %k and %l are also buggy
#
#
#-----------------------------------------------------------------------------
sub format_time {
    my ($self) = shift;
    my ($serial_number, $format_str) = @_;

    my ($date_obj) = $self -> {DATE_OBJ};
    my ($wday, $abbr_wday_str, $full_wday_str);
    my ($abbr_month_str, $full_month_str);
    my (%wday_abbr)  = %{$self -> {WDAY_ABBR}};
    my (%wday_full)  = %{$self -> {WDAY_FULL}};
    my (%month_abbr) = %{$self -> {MONTH_ABBR}};
    my (%month_full) = %{$self -> {MONTH_FULL}};
    my ($year, $month, $day, $hour, $minute, $second) = 
        $date_obj -> serial_number_to_ymdhms ($serial_number);
    $wday = $date_obj -> get_day_of_week (
        $serial_number,
        $TimeDate::WDayCode_Mon1Sun7
    );
    $abbr_wday_str = $wday_abbr {$wday};
    $full_wday_str = $wday_full {$wday};

    $abbr_month_str = $month_abbr {$month};
    $full_month_str = $month_full {$month};

    $format_str =~ s|%x|%D|g;  # should be read from locale
    $format_str =~ s|%X|%T|g;  # should be read from locale

    $format_str =~ s|%D|%m/%d/%y|g;
    $format_str =~ s|%r|%I:%M:%S %p|g;
    $format_str =~ s|%R|%H:%M|g;
    $format_str =~ s|%T|%H:%M:%S|g;
    $format_str =~ s|%c|%a %b %d %H:%M:%S %Y|g; # ???

    $format_str =~ s/%a/$abbr_wday_str/ge;
    $format_str =~ s/%A/$full_wday_str/ge;
    $format_str =~ s/(%b|%h)/$abbr_month_str/ge;
    $format_str =~ s/%B/$full_month_str/ge;
    $format_str =~ s/%C/$date_obj -> get_century ($serial_number)/ge;
    $format_str =~ s/%d/sprintf ("%.2d", $day)/ge;
    $format_str =~ s/%e/sprintf ("%2d", $day)/ge;
    $format_str =~ s/%H/sprintf ("%.2d", $hour)/ge;
    $format_str =~ s/%I/sprintf ("%.2d", ($hour % 12) == 0 ? 12 : $hour%12)/ge;
    $format_str =~ s/%j/sprintf ("%.3d",
        $date_obj -> get_day_of_year ($serial_number))/gex;
    $format_str =~ s/%k/$hour/ge;
    $format_str =~ s/%l/($hour % 12) == 0 ? 12 : $hour % 12/ge;
    $format_str =~ s/%m/sprintf ("%.2d", $month)/ge;
    $format_str =~ s/%M/sprintf ("%.2d", $minute)/ge;
    $format_str =~ s/%n/\n/g;
    $format_str =~ s/%p/($hour >= 12) ? "PM" : "AM"/ge;
    $format_str =~ s/%S/sprintf ("%.2d", $second)/ge;
    $format_str =~ s/%t/\t/g;
    $format_str =~ s/%u/$date_obj -> get_day_of_week (
        $serial_number, $TimeDate::WDayCode_Sun1Sat7)/gex;
    # %Zu, non-standard
    $format_str =~ s/%Zu/$date_obj -> get_day_of_week (
        $serial_number, $TimeDate::WDayCode_Mon1Sun7)/gex;
    # %Zw, non-standard
    $format_str =~ s/%Zw/$date_obj -> get_day_of_week (
        $serial_number, $TimeDate::WDayCode_Mon0Sun6)/gex;
    $format_str =~ s/%U/$date_obj -> get_week_number (
        $serial_number, $TimeDate::WNum_SundayBased)/gex;
    $format_str =~ s/%W/$date_obj -> get_week_number (
        $serial_number, $TimeDate::WNum_MondayBased)/gex;
    $format_str =~ s/%V/$date_obj -> get_week_number (
        $serial_number, $TimeDate::WNum_ISO)/gex;
    $format_str =~ s/%w/$date_obj -> get_day_of_week (
        $serial_number, $TimeDate::WDayCode_Sun0Sat6)/gex;
    $format_str =~ s/%y/sprintf ("%.2d", abs ($year) % 100)/ge;
    $format_str =~ s/%Y/$year/ge;
    $format_str =~ s/%Z/$ENV {"TZ"}/ge;
    $format_str =~ s/%%/%/g;

    return ($format_str);
};




1;
