package TimeDate;
#=============================================================================
#
#   Copyright (C) 1989, 1990, 1991, 1992, 1994, 1997, 2001, Matti Tukiainen
#
# MODULE NAME  : TimeDate.pm
# VERSION      : 3.21
# PURPOSE      : Date and Time management functions.
# DATE         : 2001-12-15
# UPDATES      : 1.00 The original version. (This is a combination  (01.09.91)
#                     of two old units, Date 2.00 and Time 1.10).
#                     Date range has been extended from 1.1.1901..
#                     31.12.2075 to 1.1.-4713..31.12.19999.
#                1.10 .. 2.00 Update information removed because
#                     most of this information in outdated.
#                2.00 Completely rewritten.                         (29.08.94)
#                2.01 Month_XX and WDay_XX constants added.         (31.10.94)
#                2.10 GetDaysInMonth function added.                (03.11.94)
#                2.20 GetWeekNumber function added.                 (04.11.94)
#                2.30 GetISODayOfWeek function added.  WDayISO_XX   (07.11.94)
#                     constants added.  TMonth, TWDay and TWDayISO
#                     types added.
#                2.40 GetISODayOfWeek and GetDayOfWeek functions    (15.12.94)
#                     combined.  Sunday based week number support
#                     added to the GetWeekNumber function.  WDayXX
#                     constants renamed to WDayXX1.
#                3.00 Ported to Perl running Solaris.               (03.03.97)
#                3.10 is_dst function added.                        (04.03.97)
#                3.20 get_current_gmt_time_and_date function added, (26.06.97)
#                     get_week_number now supports all strftime(3C)
#                     modes.
#                3.21 The code is now strict safe.                (2001-12-15)
#
# REFERENCES   : Compute! August 1987, p. 104.
#                Computer Language, Dec., 1990, p. 61.
#                Encyclopedia Britannica.
#                Microsoft Works Reference (1987), Microsoft Corporation,
#                  p. 229 - 255.
#                David Olson's time zone database (tzdata94.tar), available
#                  from ftp://elsie.nci.nih.gov/pub/tz*
#                Printti 18/85, p. 6
#                Printti 5/86, p. 12
#                Printti 15/85, p. 5
#                SFS-EN 28601 Tietoelementit ja esitysmuodot. Tiedonsiirto.
#                  Pivmrien ja kellonaikojen merkinttavat (ISO 8601:1988)
#                  (1993), Suomen Standardisoimisliitto SFS.
#                SFS 2671 Viikkojen numerointi (ISO/R 2015:1971) (1973?),
#                  Suomen Standardisoimislautakunta.
#                The World Almanac and Book of Facts 1992, Pharos Books,
#                  p. 291 - 296.
#                Uusi Pikkujttilinen, p. 10, 12.
#
#                Interesting information about time is available from
#                  http://tycho.usno.navy.mil/
#                  http://www.ft.uni-erlangen.de/~mskuhn/iso-time.html
#                  http://www.pip.dknet.dk/~pip10160/calendar.html
#
#                The Original algorithm for Date to/from Julian Date
#                conversion was written in Modula-2 by an unknown author.
#                I have translated it to Pascal and done some optimization.
#
# NOTES        : Serial Number = Date Number (integer part of Serial Number
#                  (Year, Month and Day)) + Time Number (fractional part of
#                  Serial number (Hour, Minute, Second and Second100)).
#                  0.0 = Jan. 1, 4713 B.C., 0:00.
#                  9,025,905 = Dec. 31, 19999 A.D.
#                Julian Day = Date Number + 0.5 (12 hours).
#                  0.0 = Jan. 1, 4713 B.C., 12:00 (noon).
#                  Was proposed by Joseph Scalizer in 1582.
#                Modified Julian Day = Julian Day - 2,400,000.5;
#                Julian Calendar = Calendar established by Julius Caesar in
#                  46 B.C., fixing the length of the year at 365 days and at
#                  366 every fourth year.
#                Gregorian Calendar = The Reformed Julian Calendar now in use,
#                  according to which the ordinary year consists of 365 days,
#                  and a leap year of 366 days occurs in every year whose
#                  number is exactly divisible by 4 except centenary years
#                  whose numbers are not exactly divisible by 400. Named
#                  after Pope Gregory XIII (1572-85).
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA @EXPORT @EXPORT_OK);
    $VERSION     = do {
        my @r = q$Revision: 3.21 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };#!!!
    @ISA         = qw (Exporter);
    @EXPORT      = qw ();
    @EXPORT_OK   = qw (
                       $WDayCode_Sun1Sat7
                       $WDayCode_Sun0Sat6
                       $WDayCode_Mon1Sun7
                       $WDayCode_Mon0Sun6

                       $WNum_MonBased
                       $WNum_SunBased 
                       $WNum_ISO
                       $WNum_MondayBased
                       $WNum_SundayBased

                       @DST_EU
                       @DST_USA
                       @DST_India
                       @DST_Nowhere

                       $DST_Relative
                       $DST_Fixed
                       $DST_None

                       $trday_Default
                       $trday_IT
                       $trday_FR
                       $trday_ES
                       $trday_PT
                       $trday_LU
                       $trday_GB
                       $trday_US
                       $trday_AU
                       $trday_CA
                       $trday_NZ
                       $trday_RU
                       $trday_SE
                       $trday_FI
                       $trday_BA
                       $trday_BG
                       $trday_HR
                       $trday_EE
                       $trday_GR1
                       $trday_GR2
                       $trday_LV
                       $trday_LT
                       $trday_MK
                       $trday_MD
                       $trday_PLATDE
                       $trday_PLRU
                       $trday_ROCatholic
                       $trday_ROOrthodox
                       $trday_RUWest
                       $trday_RUEast
                       $trday_Serbia
                       $trday_SI
    );
};

use vars @EXPORT_OK;



#-----------------------------------------------------------------------------
# Predefined constants that make it easier to write more readable code.
#-----------------------------------------------------------------------------
my ($Month_January)    = 1;
my ($Month_February)   = 2;
my ($Month_March)      = 3;
my ($Month_April)      = 4;
my ($Month_May)        = 5;
my ($Month_June)       = 6;
my ($Month_July)       = 7;
my ($Month_August)     = 8;
my ($Month_September)  = 9;
my ($Month_October)    = 10;
my ($Month_November)   = 11;
my ($Month_December)   = 12;

my (%weekday_mon1_sun7) = (
    "Monday"    => 1,
    "Tuesday"   => 2,
    "Wednesday" => 3,
    "Thursday"  => 4,
    "Friday"    => 5,
    "Saturday"  => 6,
    "Sunday"    => 7
);

#-----------------------------------------------------------------------------
# Used by get_day_of_week function.
#-----------------------------------------------------------------------------}
$WDayCode_Sun1Sat7 = 0;
$WDayCode_Sun0Sat6 = 1;
$WDayCode_Mon1Sun7 = 2;
$WDayCode_Mon0Sun6 = 3;

#-----------------------------------------------------------------------------
# Used by get_week_number function.
#-----------------------------------------------------------------------------
$WNum_MonBased = 0;     # Same as WNum_ISO, don't use this any more
$WNum_SunBased = 1;     # don't use this any more
$WNum_ISO         = 2;
$WNum_MondayBased = 3;
$WNum_SundayBased = 4;

#-----------------------------------------------------------------------------
# Used by time_coding_to_time_coding function.
#-----------------------------------------------------------------------------
my ($TCode_SerialNumber)      = 0;
my ($TCode_JulianDay)         = 1;
my ($TCode_ModifiedJulianDay) = 2;
my ($TCode_Excel)             = 3;

#-----------------------------------------------------------------------------
# Used by is_dst function.
#-----------------------------------------------------------------------------
$DST_Relative = 0;
$DST_Fixed    = 1;
$DST_None     = 2;

@DST_EU = ( # Last Sun in Mar @ 2 .. Last Sun in Oct @ 2
    $TimeDate::DST_Relative,  3, -1, 02, 00, $weekday_mon1_sun7 {"Sunday"},
    $TimeDate::DST_Relative, 10, -1, 02, 00, $weekday_mon1_sun7 {"Sunday"}
);
@DST_USA = ( # First Sun in Apr @ 2 .. Last Sun in Oct @ 2
    $TimeDate::DST_Relative,  4,  1, 02, 00, $weekday_mon1_sun7 {"Sunday"},
    $TimeDate::DST_Relative, 10, -1, 02, 00, $weekday_mon1_sun7 {"Sunday"}
);
@DST_India = ( # None .. None
    $TimeDate::DST_None, undef, undef, undef, undef, undef,
    $TimeDate::DST_None, undef, undef, undef, undef, undef
);
@DST_Nowhere = ( # Apr 15 @ 10:30 .. Oct 31 @ 23:50
    $TimeDate::DST_Fixed,  4, 15, 10, 30, undef,
    $TimeDate::DST_Fixed, 10, 31, 23, 50, undef
);

#-----------------------------------------------------------------------------
# $trday_XX constants can be used in initializing Ti18nTimeDate object.
#
# Transition day is the first date in Gregorian calendar expressed as
# a Date Number.  The Preceding day is the last date in Julian calendar.
# In the USA, for example, transition happened in Sept. 14, 1752
# (2,361,222).  The day before that day (2,361,221) was Sept. 2, 1752.
#
# In trday_XX constant, XX is an ISO 3166 alpha-2 code for a country.
# Some constants have additional information after that.
#
# Taken from The World Almanac and Book of Facts 1992, p. 291.
# In this source of data transition date for Russia is different than
# the one taken from David Olson's time zone database (tzdata94.tar).
#-----------------------------------------------------------------------------
$trday_Default    = 2299161;       # Pope Gregory XIII's order, 1582 Oct. 15.
$trday_IT         = 2299161;       # Italy, 1582 Oct. 15.
$trday_FR         = 2299161;       # France, 1582 Oct. 15.
$trday_ES         = 2299161;       # Spain, 1582 Oct. 15.
$trday_PT         = 2299161;       # Portugal, 1582 Oct. 15.
$trday_LU         = 2299161;       # Luxembourg, 1582 Oct. 15.

$trday_GB         = 2361222;       # UK, 1752 Sept. 14.
$trday_US         = 2361222;       # USA, 1752 Sept. 14.
$trday_AU         = 2361222;       # Australia, 1752 Sept. 14.
$trday_CA         = 2361222;       # (?) Canada, 1752 Sept. 14.
$trday_NZ         = 2361222;       # New Zealand, 1752 Sept. 14.

$trday_RU         = 2421639;       # Russia, Feb. 14, 1918

#----------------------------------------------------------------------------
# Taken from Uusi Pikkujttilinen, p. 10, 12.
#----------------------------------------------------------------------------
$trday_SE         = 2361390;       # Sweden, 1753 Mar. 1.
$trday_FI         = 2361390;       # Finland, 1753 Mar. 1.

#----------------------------------------------------------------------------
# Parsed from David Olson's time zone database (tzdata94.tar). Most of
# this data seems to been contributed by Paul Eggert <eggert@twinsun.com>.
# In this source of data transition date for Russia is different than
# the one taken from The World Almanac and Book of Facts 1992, p. 291.
# WABF also claims that Greece adopted Gregorian calendar in 1923.
#----------------------------------------------------------------------------
$trday_BA         = 2421671;       # Bosnia and Herzegovina, 1918 Mar. 18.
$trday_BG         = 2420816;       # Bulgaria, 1915 Nov. 14.
$trday_HR         = 2421671;       # Croatia, 1918 Mar. 18.
$trday_EE         = 2421640;       # Estonia, 1918 Feb. 15.
$trday_GR1        = 2421073;       # Greece 1, 1916 Jul. 28.
$trday_GR2        = 2422402;       # Greece 2, 1920 Mar. 18.
$trday_LV         = 2421640;       # Latvia, 1918 Feb. 15.
$trday_LT         = 2421640;       # Lithuania, 1918 Feb. 15.
$trday_MK         = 2421671;       # Macedonia, 1918 Mar. 18.
$trday_MD         = 2422036;       # Moldova, 1919 Mar. 18.
$trday_PLATDE     = 2299161;       # Austrian and German Poland, 1582 Oct. 15.
$trday_PLRU       = 2421608;       # Russian Poland, 1918 Jan. 14.
$trday_ROCatholic = 2422036;       # Catholic Romania, 1919 Mar. 18
$trday_ROOrthodox = 2422402;       # Greek Orthodox Romania, 1920 Mar. 18.
$trday_RUWest     = 2421608;       # Western Russia, 1918 Jan. 14.
$trday_RUEast     = 2422402;       # Eastern Russia, 1920 Mar. 18.
$trday_Serbia     = 2421671;       # Serbia, 1918 Mar. 18.
$trday_SI         = 2421671;       # Slovenia, 1918 Mar. 18.




#-----------------------------------------------------------------------------
# Function new
#
# Purpose  : A constructor.
#
# Input    : -
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my ($self)  = {};

    $self -> {FIRST_GREGORIAN_DATE} = $trday_Default;

    bless ($self, $class);
    return ($self);
};




#-----------------------------------------------------------------------------
# Function calendar_transition_day
#
# Purpose  : Sets FIRST_GREGORIAN_DATE variable.  Predefined trday_XX constants
#              can be used in creating an object that correctly takes into
#              account the fact that the transition from Julian calendar to
#              Gregorian calendar happened at different times in different
#              countries.
#
# Input    : -
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub calendar_transition_day {
    my $self = shift;
    if (@_) {
        $self -> {FIRST_GREGORIAN_DATE} = shift;
    };
    return ($self -> {FIRST_GREGORIAN_DATE});
};




#-----------------------------------------------------------------------------
# Procedure date_number_to_ymd
#
# Purpose  : Converts a date number into its equivalent Year, Month, and Day,
#               representation.
#
# Input    : $raw_date_number, Integer type date number for the Year, Month,
#              and a Day.
# Output   : A list of (year, month, day)
# Error
# handling : None, use is_valid_serial_number function to make sure that the
#              argument is valid.
#
#-----------------------------------------------------------------------------
sub date_number_to_ymd {
    my ($self) = shift;
    my ($raw_date_number) = @_;
    my ($date_number, $a1, $a2, $d, $day, $absolute_year, $year);
    my ($tmp_date_number, $month, $time);
    my ($res_day, $res_month, $res_year);

    $date_number = int ($raw_date_number);
    if ($date_number >= $self -> calendar_transition_day ()) {
        $a1 = $date_number - 1721120;
        $a2 = 31 * int ($a1 / 1460969);
        $a1 = $a1 % 1460969;
        $a2 = $a2 + 3 * int ($a1 / 146097);
        $a1 = $a1 % 146097;
	if ($a1 != 146096) {
            $a2 = $a2 + int ($a1 / 36524);
        }
	else {
            $a2 += 3;
        };
        $date_number = $date_number + ($a2 - 2);
    };

    $tmp_date_number = $date_number + 1524;
    $absolute_year = int ((20 * $tmp_date_number - 2442) / 7305);

    $d = int ((1461 * $absolute_year) / 4);
    $day = $tmp_date_number - $d;
    $month = int ((10000 * $day) / 306001);

    $res_day = $day - int ((306001 * $month) / 10000);
    if ($month >= 14) {
        $res_month = $month - 13;
    }
    else {
        $res_month = $month - 1;
    };
    $year = $absolute_year - 4716;
    if ($res_month <= 2) {
        $year += 1;
    };
    if ($year >= 1) {
        $res_year = $year;
    }
    else {
        $res_year = $year - 1;
    };

    return ($res_year, $res_month, $res_day);
};




#-----------------------------------------------------------------------------
# Function get_date_number
#
# Purpose  : Returns the Date Number for the date specified by a year,
#              a month, and a day.
#
# Input    : $ayear, Integer, valid range is from -4714 to 19999.
#            $amonth, Integer, valid range is from 1 to 12.
#            $aday, Integer, valid range is from 1 to 31.
# Output   : LongInt type Date Number that is in range 0..9,025,905.
# Error
# handling : None, use is_valid_date function to make sure that parameters
#              really represents some meaningful date.
#
#-----------------------------------------------------------------------------
sub get_date_number {
    my ($self) = shift;
    my ($ayear, $amonth, $aday) = @_;
    my ($absolute_year, $date_number1, $date_number2);

    if ($ayear >= 0) {
        $absolute_year = $ayear + 4716;
    }
    else {
        $absolute_year = $ayear + 4717;
    };
    if ($amonth < 3) {
        $amonth += 12;
        --$absolute_year;
        --$ayear;
    };
    $date_number1 = ((1461 * $absolute_year) >> 2) +
      (int ((153 * ($amonth + 1)) / 5)) + $aday - 1524;
    $date_number2 = (($date_number1 + 2) - int ($ayear / 100)) +
      int ($ayear / 400) - int ($ayear / 4000);
    if ($date_number2 >= $self -> calendar_transition_day ()) {
        return ($date_number2);
    }
    else {
        return ($date_number1);
    };
};




#-----------------------------------------------------------------------------
# Function get_time_number
#
# Purpose  : Returns the Time Number for the time specified by hours, minutes,
#              and seconds.
#
# Input    : hour, Integer.
#            min, Integer.
#            sec, Integer.
# Output   : Real type Time Number in which values from 0.0 to 0.9999999
#              represent times from 0:00:00 to 23:59:59.  Values
#              greater than or equal to 1.0 are treated as Serial Numbers.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_time_number {
    my ($self) = shift;
    my ($hour, $min, $sec) = @_;

    return (((60 * 60 * $hour) + (60 * $min) + $sec) / (24 * 60 * 60));
};




#-----------------------------------------------------------------------------
# Procedure time_number_to_hms
#
# Purpose  : Converts a Time Number into its equivalent Hour, Minute, and
#               Second representation.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : A list of (hour, minute, second).
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub time_number_to_hms {
    my ($self) = shift;
    my ($serial_number) = @_;
    my ($h, $m, $s);

    $serial_number = ($serial_number - int ($serial_number)) * 24 * 60 * 60;
    $serial_number = int ($serial_number + 0.5 * ($serial_number <=> 0));

    $h = int ($serial_number / 3600);
    $m = int (60 * (($serial_number - ($h * 3600)) / 3600));
    $s = $serial_number - ($h * 3600 + $m * 60);
    $s = int ($s + 0.5 * ($s <=> 0));

#    if ($s >= 60) {
#        $s = 0;
#        $m++;
#        if ($m >= 60) {
#            $m = 0;
#            $h++;
#        };
#    };

    return ($h, $m, $s);
};




#-----------------------------------------------------------------------------
# Function get_hour
#
# Purpose  : Returns the number of the hour represented by the serial number.
#
# Input    : $serial_number, Real type Serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : Integer type number of the hour ranging from 0 through 23.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_hour {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (($self -> time_number_to_hms ($serial_number))[0]);
};




#-----------------------------------------------------------------------------
# Function get_minute
#
# Purpose  : Returns the number of the minute represented by the serial
#              number.
#
# Input    : $serial_number, Real type Serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : Integer type number of the minute ranging from 0 through 59.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_minute {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (($self -> time_number_to_hms ($serial_number))[1]);
};




#-----------------------------------------------------------------------------
# Function get_second
#
# Purpose  : Returns the number of the second represented by the serial
#              number.
#
# Input    : $serial_number, Real type Serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : Integer type number of the minute ranging from 0 through 59.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_second {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (($self -> time_number_to_hms ($serial_number))[2]);
};




#-----------------------------------------------------------------------------
# Function serial_number_to_ymdhms
#
# Purpose  : Converts a serial number into its equivalent Year, Month, Day,
#               Hour, Minute, and Second representation.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : A list of (year, month, day, hour, minute, second).
# Error
# handling : None, use is_valid_serial_number function to make sure that the
#              argument is valid.
#
#-----------------------------------------------------------------------------
sub serial_number_to_ymdhms {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (
        $self -> date_number_to_ymd ($serial_number),
        $self -> time_number_to_hms ($serial_number)
    );
};



#-----------------------------------------------------------------------------
# Function ymdhms_to_serial_number
#
# Purpose  : Converts Year, Month, Day, Hour, Minute, and Second to Serial
#              Number.
#
# Input    : $year, $month, $day, $hour, $min, $sec, Integer.
# Output   : Real type Serial Number.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub ymdhms_to_serial_number {
    my ($self) = shift;
    my ($year, $month, $day, $hour, $min, $sec) = @_;

    return (
        $self -> get_date_number ($year, $month, $day) +
        $self -> get_time_number ($hour, $min, $sec)
    );
};




#-----------------------------------------------------------------------------
# Function get_century
#
# Purpose  : Returns the number of the century represented by the serial
#              number.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second
# Output   : Integer type number of the century ranging from -47 to 199.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_century {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (int ((($self -> date_number_to_ymd ($serial_number))[0]) / 100));
};




#-----------------------------------------------------------------------------
# Function get_year
#
# Purpose  : Returns the number of the year represented by the serial
#              number.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second
# Output   : Integer type number of the year ranging from -4713 to 19999.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_year {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (($self -> date_number_to_ymd (int ($serial_number)))[0]);
};




#-----------------------------------------------------------------------------
# Function get_month
#
# Purpose  : Returns the number of the month represented by the serial
#              number.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second
# Output   : Integer type number of the ranging from 1 to 12.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_month {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (($self -> date_number_to_ymd ($serial_number))[1]);
};




#-----------------------------------------------------------------------------
# Function get_day
#
# Purpose  : Returns the number of the day represented by the serial number.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second
# Output   : Integer type number of the day ranging from 1 through 31.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_day {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (($self -> date_number_to_ymd ($serial_number))[2]);
};




#-----------------------------------------------------------------------------
# Function get_day_of_week
#
# Purpose  : Returns the day of week represented by the serial number.
#             The day is given as an integer value.  Four different kind of
#             coding schemes are supported.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
#            $day_of_week_coding, Integer type parameter that determines
#              the coding of return value.  Use $TimeDate::WDayCode_*
#              constants.
# Output   :Integer type number of the day of week ranging from:
#              1 (Sun)..7 (Sat), when $day_of_week_coding = WDayCode_Sun1Sat7,
#              0 (Sun)..6 (Sat), when $day_of_week_coding = WDayCode_Sun0Sat6,
#              1 (Mon)..7 (Sun), when $day_of_week_coding = WDayCode_Mon1Sun7,
#              0 (Mon)..6 (Sun), when $day_of_week_coding = WDayCode_Mon0Sun6.
# Error
# handling : None.
# Notes    : Results of this function may be unreliable on pre Gregorian
#              calendar era because of weak robustness of calendars in use
#              at that time.
#
#-----------------------------------------------------------------------------
sub get_day_of_week {
    my ($self) = shift;
    my ($serial_number, $day_of_week_coding) = @_;
    my ($dow);

    if ($day_of_week_coding == $WDayCode_Sun1Sat7) {
        $dow = ((int ($serial_number) + 1) % 7) + 1
    }
    elsif ($day_of_week_coding == $WDayCode_Sun0Sat6) {
        $dow = (int ($serial_number) + 1) % 7
    }
    elsif ($day_of_week_coding == $WDayCode_Mon1Sun7) {
        $dow = (int ($serial_number) % 7) + 1
    }
    elsif ($day_of_week_coding == $WDayCode_Mon0Sun6) {
        $dow = (int ($serial_number)) % 7
    };

    return ($dow);
};




#-----------------------------------------------------------------------------
# Function get_day_of_year
#
# Purpose  : Returns the ordinal number of the particular day of a year
#              represented by the serial number.  The first day of any year
#              is 001 and the subsequent days are numbered in ascending
#              sequence.  The last day of any common year is 365 and 366 for
#              any leap year.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : Integer type number of the day of year ranging from 1 (Jan. 1)
#              through 365 (or 366) (Dec. 31).
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_day_of_year {
    my ($self) = shift;
    my ($serial_number) = @_;

    return (
        int ($serial_number) -
        $self -> get_date_number (
            ($self -> date_number_to_ymd ($serial_number))[0],
            1,
            1
        ) + 1
    );
};




#-----------------------------------------------------------------------------
# Function get_days_in_month
#
# Purpose  : Returns the number of days in a given year and month.
#
# Input    : $year, Integer, a year, valid range is from -4713 to 19999.
#            $month, Byte, a month, valid range is from 1 to 12.
# Output   : Byte, Days in month.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_days_in_month {
    my ($self) = shift;
    my ($year, $month) = @_;
    my ($next_year, $next_month);

    $next_year = $year;
    $next_month = $month + 1;
    if ($next_month > $Month_December) {
        $next_month = $Month_January;
        $next_year = $year + 1;
    };

    return (
        $self -> get_date_number ($next_year, $next_month, 1) -
        $self -> get_date_number ($year, $month, 1)
    );
};




#-----------------------------------------------------------------------------
# Function time_coding_to_time_coding
#
# Purpose  : Converts a Serial Number, Julian Day, Modified Julian Day or
#              Excel (spreadsheet) coded number to another time coding.
#
# Input    : $from_time_coding, integer type variable (see $TCode_XXX
#              constants) that describes the coding of the third parameter,
#              $coded_time.
#            $to_time_coding, integer type variable (see $TCode_XXX constants)
#              that describes the coding in which the result is wanted to be.
#            $coded_time, Real type $from_time_coding coded number to be
#              converted.
# Output   : Real type Serial Number, Julian Day, Modified Julian Day or
#              Excel (spreadsheet) serial number.
# Error
# handling : None.
# Notes    : Many spreadsheet programs (at least MS-Excel 2.0 - 5.0 and
#              MS-Works 1.0) have a leap year bug in their date functions.
#              According to these programs year 1900 was a leap year and thus
#              had 29 days in February.  This is not true and therefore
#              non-existed February 29 (Day#60) is trated as if it was March
#              1 (Day#61).
#            Excel 2.0 - 5.0 spreadsheet programs are only capable of
#              handlind dated in the range from 1.1.1900 (Day#1) through
#              31.12.2078 (Day#65,381).
#
# Refs     : The World Almanac and Book of Facts 1992, p. 296.
#            Encyclopedia Britannica.
#
#-----------------------------------------------------------------------------
sub time_coding_to_time_coding {
    my ($self) = shift;
    my ($from_time_coding, $to_time_coding, $coded_time) = @_;
    my ($result);
    #                      SerialNumber, JulianDay, ModJulianDay,   Excel 
    my (@serial_number) = (0.0,         -0.5,        -2400000.0,   -2415019.0);
    my (@julian_day)    = (0.5,          0.0,        -2400000.5,   -2415018.5);
    my (@mod_julian_day)= (2400000.0,    2400000.5,   0.0,         -15019.0);
    my (@excel) =         (2415019.0,    2415018.5,   15019.0,      0.0);
    my (@conversion_table) = (
        \@serial_number,
        \@julian_day,
        \@mod_julian_day,
        \@excel
    );

    $result = $coded_time +
        $conversion_table [$from_time_coding][$to_time_coding];

    if (($from_time_coding = $TCode_Excel) && ($coded_time < 61)) {
        $result = $result + 1;
    };
    if (($to_time_coding = $TCode_Excel) && ($result < 61)) {
        $result = $result - 1;
    };

    return ($result);
};




#-----------------------------------------------------------------------------
# Function get_n_th_weekday
#
# Purpose  : Returns the serial number for the weekday's date n weeks
#              ago/ahead of time given as a parameter.  Ie this function
#              makes it easy to to give answers to questions like "when will
#              be the last Sunday in October" or "when was the second
#              Tuesday in March".
#
# Input    : $start_date_serial_number, Real type serial number for the Year,
#              Month, Day, Hour, Minute, and Second.
#            $count, Integer type parameter that descibes the number of weeks.
#              Positive numbers look forward, and negative backward.
#            $weekday, Integer, 1 (Mon)..7 (Sun)
#
# Output   : Boolean, 1 if AYear is leap year, 0 if it isn't.
# Error
# handling : None.
# Examples : Today = Monday, 1997, 9, 1:
#            First Monday in Sept.?
#              get_n_th_weekday (xxx (1997, 9, 1), 1, 1) =>  1997.9.1
#            First Sunday in Sept.?
#              get_n_th_weekday (xxx (1997, 9, 1), 1, 7) =>  1997.9.7
#            Last Monday in Aug.?
#              get_n_th_weekday (xxx (1997, 8, 31), -1, 1) => 1997.8.25
#            Last Sunday in Aug.?
#              get_n_th_weekday (xxx (1997, 8, 31), -1, 7) => 1997.8.31
#            5th Sunday in Mar 1997.?
#              get_n_th_weekday (xxx (1997, 3, 1), 5, 7) => 1997.3.30
#
#-----------------------------------------------------------------------------
sub get_n_th_weekday {
    my ($self) = shift;
    my ($start_date_serial_number, $count, $weekday) = @_;
    my ($found, $found_count);

    $found_count = 0;
    $count = int ($count);

    if ($count != 0) {
        do {
            if ($self -> get_day_of_week ($start_date_serial_number,
                $WDayCode_Mon1Sun7) == $weekday) {
                $found_count++;
            };
            $found = ($found_count == abs ($count));
            if (not ($found)) {
                ($count > 0) ?
                    $start_date_serial_number++ : $start_date_serial_number--;
            };
        } until ($found);
    };

    return ($start_date_serial_number);
};




#-----------------------------------------------------------------------------
# Function get_easter
#
# Purpose  : Returns the Serial Number of the Easter (Sunday following the
#              first full moon after the Spring equinox) for the specified
#              year.
#
# Input    : $year, integer, a year to be checked, valid range is from 0 to
#              2900.
# Output   : Integer type Serial Number.
# Error
# handling : None.
# Refs     : Computer Language, Dec., 1990, p. 61.
#
#-----------------------------------------------------------------------------
sub get_easter {
    my ($self) = shift;
    my ($year) = @_;
    my ($a, $b, $c, $d, $e, $f, $g, $h, $i, $k, $l, $m, $n, $p, $day, $month);

    if ($year >= ($self -> get_year ($self -> calendar_transition_day ()) +1)){
        $a = $year % 19;
        $b = int ($year / 100);
        $c = $year % 100;
        $d = int ($b / 4);
        $e = $b % 4;
        $f = int (($b + 8) / 25);
        $g = int (($b - $f + 1) / 3);
        $h = (19 * $a + $b - $d - $g + 15) % 30;
        $i = int ($c / 4);
        $k = $c % 4;
        $l = (32 + 2 * $e + 2 * $i - $h - $k) % 7;
        $m = int (($a + 11 * $h + 22 * $l) / 451);
        $n = int (($h + $l - 7 * $m + 114) / 31);
        $p = ($h + $l - 7 * $m + 114) % 31;
        $month = $n;
        $day = $p + 1;
    }
    else {
        $a = $year % 4;
        $b = $year % 7;
        $c = $year % 19;
        $d = (19 * $c + 15) % 30;
        $e = (2 * $a + 4 * $b - $d + 34) % 7;
        $f = int (($d + $e + 114) / 31);
        $g = ($d + $e + 114) % 31;
        $month = $f;
        $day = $g + 1;
    };

    return ($self -> get_date_number ($year, $month, $day));
};




#-----------------------------------------------------------------------------
# Function is_leap_year
#
# Purpose  : Checks if the particular year is a leap year.
#
# Input    : $year, Integer, a year to be checked, valid range is from -4714
#              to 19999.
# Output   : Boolean, 1 if $year is leap year, 0 if it isn't.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub is_leap_year {
    my ($self) = shift;
    my ($year) = @_;

    # Substraction is zero based; Common years has 365 - 1 = 364 days
    # and a leap year has 366 - 1 = 365 days.
    return (
        ($self -> get_date_number ($year, 12, 31) -
            $self -> get_date_number ($year, 1, 1)) == 365
    );
};




#-----------------------------------------------------------------------------
# Function is_valid_serial_number
#
# Purpose  : Returns True if the Serial Number is in valid range (0..9025905).
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.
# Output   : True if the Serial Number is in valid range, False if it isn't.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub is_valid_serial_number {
    my ($self) = shift;
    my ($serial_number) = @_;
    my ($max_julian_date) = 9025905; # Dec, 31. 19,999

    return (
        (($serial_number >= 0) && ($serial_number < ($max_julian_date + 1)))
    );
};




#-----------------------------------------------------------------------------
# Function is_valid_date
#
# Purpose  : Returns True if the date specified by a year, a month, and a
#              day is valid.
#
# Input    : $year, Integer.
#            $month, Integer.
#            $day, Integer.
# Output   : True if parameters represent a legal date, False if they doesn't.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub is_valid_date {
    my ($self) = shift;
    my ($year, $month, $day) = @_;
    my ($yy, $mm, $dd);
    my ($ok);

    $ok = (defined ($year) && defined ($month) && defined ($day));
    if ($ok) {
        ($yy, $mm, $dd) = $self -> date_number_to_ymd (
            $self -> get_date_number ($year, $month, $day)
	);
        $ok = (($yy == $year) && ($mm == $month) && ($dd == $day));
    };

    return ($ok);

};




#-----------------------------------------------------------------------------
# Function is_valid_time
#
# Purpose  : Returns True if the time (00:00:00 - 23:59:59) specified by a
#              hour, a min, and a sec is valid.
#
# Input    : $hour, Integer.
#            $min, Integer.
#            $sec, Integer.
# Output   : True if parameters represent a legal time, False if they doesn't.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub is_valid_time {
    my ($self) = shift;
    my ($hour, $min, $sec) = @_;
    my ($hh, $mm, $ss);
    my ($ok);

    $ok = (defined ($hour) && defined ($min) && defined ($sec));
    if ($ok) {
        ($hh, $mm, $ss) = $self -> time_number_to_hms (
            $self -> get_time_number ($hour, $min, $sec)
	);
        $ok = (($hh == $hour) && ($mm == $min) && ($ss == $sec));
    };

    return ($ok);
};




#-----------------------------------------------------------------------------
# Function is_dst
#
# Purpose  : Returns True if the time and date given as Serial Number lies
#              in Daylight Savings Time period on a given area.
#
# Input    : $start_dst_format, Integer, specifies how the following
#              values should be interpreted.  Use $DST_XXX constants.
#            $start/end_month, Integer, Month when the dst begins/ends.
#              Use 0 or "undef" if the dst is not used ($DST_None).
#            $start/end_day, Integer.  1 = first, 2 = second, ..., -1 = last
#              ($start/end_weekday in month) if $start_dst_format =
#              $DST_Relative, or the exact day in that $start/end_month if
#              $DST_Fixed.
#            $start/end_hour, min, Integer, hour, min.
#            $start/end_weekday, Interger, monday = 1 .. sun = 7 coded
#              weekday.  Used only if $start_dst_format = $DST_Relative
# Output   : Boolean
# Error
# handling : None.
# Examples : $date -> is_dst (
#                $date -> ymdhms_to_serial_number (1997, 3, 23, 23, 00, 00),
#                @TimeDate::DST_EU)
#            $date -> is_dst (
#                $date -> ymdhms_to_serial_number (1997, 3, 23, 23, 00, 00),
#                $TimeDate::DST_Relative,  3, -1, 02, 00, 7,
#                $TimeDate::DST_Relative, 10, -1, 02, 00, 7)
#
#-----------------------------------------------------------------------------
sub is_dst {
    my ($self) = shift;
    my ($serial_number,
        $start_dst_format, $start_month, $start_day, $start_hour, $start_min,
        $start_rel_weekday,
        $end_dst_format, $end_month, $end_day, $end_hour, $end_min,
        $end_rel_weekday
    ) = @_;
    my ($in_dst);
    my ($dst_start, $dst_end, $year);

    $year = $self -> get_year ($serial_number);

    if ($start_dst_format == $DST_Relative) {
        $dst_start = $self -> get_n_th_weekday (
            $self -> get_date_number (
                $year,
                $start_month,
                ($start_day > 0) ?
                    1 :
                    $self -> get_days_in_month ($year, $start_month)
            ),
            $start_day,
            $start_rel_weekday
	) +
        $self -> get_time_number ($start_hour, $start_min, 0);
    }
    elsif ($start_dst_format == $DST_Fixed) {
        $dst_start = $self -> ymdhms_to_serial_number (
            $year, $start_month, $start_day,
            $start_hour, $start_min, 0
        );
    }
    else {
        $dst_start = 0;
    };

    if ($end_dst_format == $DST_Relative) {
        $dst_end = $self -> get_n_th_weekday (
            $self -> get_date_number (
                $year,
                $end_month,
                ($end_day > 0) ?
                    1 :
                    $self -> get_days_in_month ($year, $end_month)
            ),
            $end_day,
            $end_rel_weekday
	) +
        $self -> get_time_number ($end_hour, $end_min, 0);
    }
    elsif ($end_dst_format == $DST_Fixed) {
        $dst_end = $self -> ymdhms_to_serial_number (
            $year, $end_month, $end_day,
            $end_hour, $end_min, 0
        );
    }
    else {
        $dst_end = 0;
    };

    return (($serial_number >= $dst_start) && ($serial_number < $dst_end));
};




#-----------------------------------------------------------------------------
# Function get_week_number
#
# Purpose  : Returns the number of the week represented by the serial number.
#              ISO, Sunday, and Monday (as the first day of the week) based
#              week numbering systems are supported.
#
# Input    : $serial_number, Real type serial number for the Year, Month, Day,
#              Hour, Minute, and Second.  Valid range for Monday based week
#              numbers is from -4712 to 19998, and from -4713 to 19999 for
#              Sunday based week numbers.
#            $week_numbering, Integer type parameter that determines
#              which numbering scheme should be used.
#              $TimeDate::WNum_ISO : Monday is the first day of the week.  If
#                the week containing 1 January has four or more days in the
#                new year, then it is considered week 1; otherwise, it is week
#                53 of the previous  year, and the next week is week 1.
#              $TimeDate::WNum_MondayBased : Week number 1 is the first week
#                in January starting with a Monday.  Week number 0 contains
#                those days before the first Monday in January.
#              $TimeDate::WNum_SundayBased : Week number 1 is the first week
#                in January starting with a Sunday.  Week number 0 contains
#                those days before the first Sunday in January.
#
# Output   : Byte type number of the week ranging from 1 to 53 (ISO), 0 to 53
#              (Sunday based) or from 0 to 53 (Monday based).
# Error
# handling : None.
# Notes    : 
# Refs     : SFS-EN 28601, p. 6.
#            SFS 2671, p. 1.
#            strftime(3C)
#
#-----------------------------------------------------------------------------
sub get_week_number {
    my ($self) = shift;
    my ($serial_number, $week_numbering) = @_;

                              # Week begins on Monday...
    my ($first_monday);       # Date Number of the first Mon, this year
    my ($next_first_monday);  # Date Number of the first Mon, next year
    my ($prev_first_monday);  # Date Number of the first Mon, prev year
    my ($this_year);
    my ($week_number);
    my ($days_since_1st_day);

    $serial_number = int ($serial_number);
    if (($week_numbering == $WNum_ISO) || ($week_numbering == $WNum_MonBased)){
        # The first calendar week of the year is the one that includes 4 Jan.
        $this_year = ($self -> date_number_to_ymd ($serial_number))[0];
        $first_monday = $self -> get_date_number ($this_year, 1, 4) -
            $self -> get_day_of_week (
                $self -> get_date_number ($this_year, 1, 4),
                $WDayCode_Mon0Sun6
            );
        $next_first_monday = $self -> get_date_number ($this_year + 1, 1, 4) -
            $self -> get_day_of_week (
                $self -> get_date_number ($this_year + 1, 1, 4),
                $WDayCode_Mon0Sun6
            );
        $prev_first_monday = $self -> get_date_number ($this_year - 1, 1, 4) -
            $self -> get_day_of_week (
                $self -> get_date_number ($this_year - 1, 1, 4),
                $WDayCode_Mon0Sun6
            );
        if ($serial_number >= $next_first_monday) {
            $first_monday = $next_first_monday
	}
	elsif ($serial_number < $first_monday) {
            $first_monday = $prev_first_monday
	};

        $week_number = int (($serial_number - $first_monday) / 7) + 1;
    }
    elsif (($week_numbering == $WNum_MondayBased) ||
           ($week_numbering == $WNum_SundayBased)) {
	$days_since_1st_day =
            int ($serial_number) -
            $self -> get_n_th_weekday (
	        $self -> get_date_number (
		    $self -> get_year ($serial_number),
                    1,
                    1
                ),
                1,
		($week_numbering == $WNum_SundayBased) ?
		    $weekday_mon1_sun7 {"Sunday"} :
		    $weekday_mon1_sun7 {"Monday"}
            );
	$week_number = ($days_since_1st_day >= 0) ?
            int ($days_since_1st_day / 7) + 1:
            0;
    }
    else {
        $week_number = int ((
            ($self -> get_day_of_year ($serial_number) - 1) + 7 -
            $self -> get_day_of_week (
                $serial_number,
                $WDayCode_Sun0Sat6
            )) / 7);
    };

    return ($week_number);
};




#-----------------------------------------------------------------------------
# Function local_time_to_local_time
#
# Purpose  : Returns the local time (and date) in another time zone expressed
#              as a Serial Number.
#
# Input    : $from_serial_number, Real type serial number for the Year, Month,
#              Day, Hour, Minute, and Second in "source" time zone.
#            $from_utc_time_difference, Integer, the difference between local
#              time in the "source" zone and the Coordinated Universal Time
#              (UTC) in minutes. get_time_zone_diff_to_utc function can be
#              used to get this value.
#            $to_utc_time_difference, Integer, the difference between local
#              time in the "target" zone and the Coordinated Universal Time
#              (UTC) in minutes. get_time_zone_diff_to_utc function can be
#              used to get this value.
# Output   : Real type Serial Number.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub local_time_to_local_time {

    my ($self) = shift;
    my (
        $from_serial_number,
        $from_utc_time_diff,
        $to_utc_time_diff
    ) = @_;
    my ($utc_time, $zone_diff);

    if ((($to_utc_time_diff > 0) && ($from_utc_time_diff < 0)) ||
        (($to_utc_time_diff < 0) && ($from_utc_time_diff > 0))) {
        $zone_diff = abs ($to_utc_time_diff) + abs ($from_utc_time_diff);
    }
    else {
        $zone_diff = abs ($to_utc_time_diff) - abs ($from_utc_time_diff);
    };

    if ($to_utc_time_diff > $from_utc_time_diff) {
        $utc_time = $from_serial_number +
            $self -> get_time_number (0, abs ($zone_diff), 0);
    }
    else {
        $utc_time = $from_serial_number -
            $self -> get_time_number (0, abs ($zone_diff), 0);
    }

    return ($utc_time);
};




#-----------------------------------------------------------------------------
# Function get_current_time_and_date
#
# Purpose  : Returns the Serial Number for the current time and date set in
#             the operating system.
#
# Input    : -
# Output   : Real type Serial Number for the current time and date.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_current_time_and_date {
    my ($self) = shift;
    my (@time_now) = localtime (time);
    my ($sec)   = $time_now [0];
    my ($min)   = $time_now [1];
    my ($hour)  = $time_now [2];
    my ($day)   = $time_now [3];
    my ($month) = $time_now [4] + 1;
    my ($year)  = $time_now [5] + 1900;

    return (
        $self -> get_date_number ($year, $month, $day) +
        $self -> get_time_number ($hour, $min, $sec)
    );
};




#-----------------------------------------------------------------------------
# Function get_current_gmt_time_and_date
#
# Purpose  : Returns the Serial Number for the current time and date set in
#             the operating system.
#
# Input    : -
# Output   : Real type Serial Number for the current time and date.
# Error
# handling : None.
#
#-----------------------------------------------------------------------------
sub get_current_gmt_time_and_date {
    my ($self) = shift;
    my (@time_now) = gmtime (time);
    my ($sec)   = $time_now [0];
    my ($min)   = $time_now [1];
    my ($hour)  = $time_now [2];
    my ($day)   = $time_now [3];
    my ($month) = $time_now [4] + 1;
    my ($year)  = $time_now [5] + 1900;

    return (
        $self -> get_date_number ($year, $month, $day) +
        $self -> get_time_number ($hour, $min, $sec)
    );
};




#-----------------------------------------------------------------------------
# Function get_time_zones
#
# Purpose  : Returns an assosiated array of some of the most common time
#              zone names and their corresponding difference from UTC time
#              in minutes.
#
# Input    : -
# Output   : An assosiated array.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub get_time_zones {
    my ($self) = shift;
    my (%time_zone_hash) = (
        "gmt"  =>   0,           # gmt  Greenwich
        "utc"  =>   0 * 60,      # utc
        "ut"   =>   0 * 60,      # ut

        "nzst" =>  12 * 60,      # nzst New Zealand
        "jst"  =>   9 * 60,      # jst  Japan
        "kst"  =>   9 * 60,      # kst  Korea
        "ist"  =>   5 * 60 + 30, # ist  India
        "eet"  =>   2 * 60,      # eet  Eastern Europe
        "eed"  =>   3 * 60,      # eed  Eastern European Daylight Time
        "cet"  =>   1 * 60,      # cet  Central Europe
        "met"  =>   1 * 60,      # met  Middle Europe
        "med"  =>   2 * 60,      # med  Middle European Daylight Time
        "wet"  =>   0 * 60,      # wet  Western Europe
        "nst"  =>  -3 * 60 - 30, # nst  Newfoundland
        "ast"  =>  -4 * 60,      # ast  Atlantic
        "est"  =>  -5 * 60,      # est  Eastern
        "cst"  =>  -6 * 60,      # cst  Central
        "mst"  =>  -7 * 60,      # mst  Mountain
        "pst"  =>  -8 * 60,      # pst  Pacific
        "akst" =>  -9 * 60,      # akst Alaska
        "hast" => -10 * 60,      # hast Hawaii-Aleutian
        "hst"  => -10 * 60,      # hst  Hawaii
        "sst"  => -11 * 60,      # sst  Samoa

        "nzdt" =>  13 * 60,      # nzdt New Zealand
        "kdt"  =>  10 * 60,      # kdt  Korea
        "bst"  =>   1 * 60,      # bst  Britain
        "ndt"  =>  -2 * 60 - 30, # ndt  Newfoundland (DDST)
        "adt"  =>  -3 * 60,      # adt  Atlantic
        "edt"  =>  -4 * 60,      # edt  Eastern
        "cdt"  =>  -5 * 60,      # cdt  Central
        "mdt"  =>  -6 * 60,      # mdt  Mountain
        "pdt"  =>  -7 * 60,      # pdt  Pacific
        "akdt" =>  -8 * 60,      # akdt Alaska
        "hadt" =>  -9 * 60,      # hadt Hawaii-Aleutian

        "lt"   =>   0,           # lt   local time
        "dst"  =>   1 * 60,      # dst  daylight savings time
        "ddst" =>   2 * 60       # ddst double dst
    );

    return (%time_zone_hash);
};




#-----------------------------------------------------------------------------
# Function get_time_zone_diff_to_utc
#
# Purpose  : Returns the difference of time between UTC time and some
#              other timezone in minutes.
#
# Input    : $tz_str, String, Name of the time zone.
# Output   : Integer
# Error
# handling : Returns 0 if the time zone $tz_str doesn't exist.
#
#-----------------------------------------------------------------------------
sub get_time_zone_diff_to_utc {
    my ($self) = shift;
    my ($tz_str) = @_;
    my (%tz_hash) = $self -> get_time_zones ();
    my ($diff) = 0;

    if (exists ($tz_hash {$tz_str})) {
        $diff = $tz_hash {$tz_str}
    };

    return ($diff);
};




1;
